@@ -1,7 +1,7 @@
 /*
  * $Header$
- * $Revision$
- * $Date$
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  * ====================================================================
  *
  *  Copyright 2002-2004 The Apache Software Foundation
@@ -89,7 +89,8 @@ public class ClientApp {
         
         public HttpClientFrame() {            
             client = new HttpClient(new MultiThreadedHttpConnectionManager());
-            client.setConnectionTimeout(30000);
+            client.getHttpConnectionManager().
+                getParams().setConnectionTimeout(30000);
 
             JPanel panInput = new JPanel(new FlowLayout());
 
