@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/examples/CookieDemoApp.java,v 1.10.2.1 2004/02/22 18:21:12 olegk Exp $
- * $Revision: 1.10.2.1 $
- * $Date: 2004/02/22 18:21:12 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/examples/CookieDemoApp.java,v 1.14 2004/02/22 18:08:45 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  * ====================================================================
  *
  *  Copyright 2002-2004 The Apache Software Foundation
@@ -76,24 +76,27 @@ public class CookieDemoApp {
         Cookie mycookie = new Cookie(".foobar.com", "mycookie", "stuff", "/", null, false);
         // and then added to your HTTP state instance
         initialState.addCookie(mycookie);
+
+        // Get HTTP client instance
+        HttpClient httpclient = new HttpClient();
+        httpclient.getHttpConnectionManager().
+            getParams().setConnectionTimeout(30000);
+        httpclient.setState(initialState);
+
         // RFC 2101 cookie management spec is used per default
         // to parse, validate, format & match cookies
-        initialState.setCookiePolicy(CookiePolicy.RFC2109);
+        httpclient.getParams().setCookiePolicy(CookiePolicy.RFC_2109);
         // A different cookie management spec can be selected
         // when desired
 
-//      initialState.setCookiePolicy(CookiePolicy.NETSCAPE_DRAFT);
+        //httpclient.getParams().setCookiePolicy(CookiePolicy.NETSCAPE);
         // Netscape Cookie Draft spec is provided for completeness
         // You would hardly want to use this spec in real life situations
-//      initialState.setCookiePolicy(CookiePolicy.COMPATIBILITY);
+        //httppclient.getParams().setCookiePolicy(CookiePolicy.BROWSER_COMPATIBILITY);
         // Compatibility policy is provided in order to mimic cookie
         // management of popular web browsers that is in some areas 
         // not 100% standards compliant
 
-        // Get HTTP client instance
-        HttpClient httpclient = new HttpClient();
-        httpclient.setConnectionTimeout(30000);
-        httpclient.setState(initialState);
         // Get HTTP GET method
         GetMethod httpget = new GetMethod(strURL);
         // Execute HTTP GET
