@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/examples/PostXML.java,v 1.10.2.1 2004/02/22 18:21:12 olegk Exp $
- * $Revision: 1.10.2.1 $
- * $Date: 2004/02/22 18:21:12 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/examples/PostXML.java,v 1.13 2004/05/12 20:43:53 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  * ====================================================================
  *
  *  Copyright 2002-2004 The Apache Software Foundation
@@ -32,7 +32,7 @@ import java.io.File;
 import java.io.FileInputStream;
 
 import org.apache.commons.httpclient.HttpClient;
-import org.apache.commons.httpclient.methods.EntityEnclosingMethod;
+import org.apache.commons.httpclient.methods.InputStreamRequestEntity;
 import org.apache.commons.httpclient.methods.PostMethod;
 
 /**
@@ -78,17 +78,12 @@ public class PostXML {
         PostMethod post = new PostMethod(strURL);
         // Request content will be retrieved directly 
         // from the input stream
-        post.setRequestBody(new FileInputStream(input));
         // Per default, the request content needs to be buffered
         // in order to determine its length.
         // Request body buffering can be avoided when
-        // = content length is explicitly specified
-        // = chunk-encoding is used
-        if (input.length() < Integer.MAX_VALUE) {
-            post.setRequestContentLength((int)input.length());
-        } else {
-            post.setRequestContentLength(EntityEnclosingMethod.CONTENT_LENGTH_CHUNKED);
-        }
+        // content length is explicitly specified
+        post.setRequestEntity(new InputStreamRequestEntity(
+            new FileInputStream(input), input.length()));
         // Specify content type and encoding
         // If content encoding is not explicitly specified
         // ISO-8859-1 is assumed
@@ -96,13 +91,16 @@ public class PostXML {
         // Get HTTP client
         HttpClient httpclient = new HttpClient();
         // Execute request
-        int result = httpclient.executeMethod(post);
-        // Display status code
-        System.out.println("Response status code: " + result);
-        // Display response
-        System.out.println("Response body: ");
-        System.out.println(post.getResponseBodyAsString());
-        // Release current connection to the connection pool once you are done
-        post.releaseConnection();
+        try {
+            int result = httpclient.executeMethod(post);
+            // Display status code
+            System.out.println("Response status code: " + result);
+            // Display response
+            System.out.println("Response body: ");
+            System.out.println(post.getResponseBodyAsString());
+        } finally {
+            // Release current connection to the connection pool once you are done
+            post.releaseConnection();
+        }
     }
 }
