@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/examples/TrivialApp.java,v 1.13.2.1 2004/02/22 18:21:12 olegk Exp $
- * $Revision: 1.13.2.1 $
- * $Date: 2004/02/22 18:21:12 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/examples/TrivialApp.java,v 1.18 2004/06/12 22:47:23 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  *
  * ====================================================================
  *
@@ -37,6 +37,7 @@ import org.apache.commons.httpclient.HttpClient;
 import org.apache.commons.httpclient.HttpException;
 import org.apache.commons.httpclient.HttpMethod;
 import org.apache.commons.httpclient.UsernamePasswordCredentials;
+import org.apache.commons.httpclient.auth.AuthScope;
 import org.apache.commons.httpclient.methods.GetMethod;
 
 /**
@@ -78,11 +79,12 @@ public class TrivialApp
         HttpClient client = new HttpClient();
 
         //establish a connection within 5 seconds
-        client.setConnectionTimeout(5000);
+        client.getHttpConnectionManager().
+            getParams().setConnectionTimeout(5000);
 
         //set the default credentials
         if (creds != null) {
-            client.getState().setCredentials(null, null, creds);
+            client.getState().setCredentials(AuthScope.ANY, creds);
         }
 
         String url = args[0];
@@ -91,7 +93,6 @@ public class TrivialApp
         //create a method object
             method = new GetMethod(url);
             method.setFollowRedirects(true);
-            method.setStrictMode(false);
         //} catch (MalformedURLException murle) {
         //    System.out.println("<url> argument '" + url
         //            + "' is not a valid URL");
@@ -136,7 +137,6 @@ public class TrivialApp
 
         //clean up the connection resources
         method.releaseConnection();
-        method.recycle();
 
         System.exit(0);
     }
