@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/HttpConnectionManager.java,v 1.15.2.1 2004/02/22 18:21:13 olegk Exp $
- * $Revision: 1.15.2.1 $
- * $Date: 2004/02/22 18:21:13 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/HttpConnectionManager.java,v 1.24 2004/07/05 22:46:58 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  *
  * ====================================================================
  *
@@ -25,20 +25,21 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient;
 
+import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
+
 /**
  * An interface for classes that manage HttpConnections.
  * 
  * @see org.apache.commons.httpclient.HttpConnection
  * @see org.apache.commons.httpclient.HttpClient#HttpClient(HttpConnectionManager)
  *
- * @author Unascribed
+ * @author Michael Becke
  * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
+ * @author <a href="mailto:oleg@ural.ru">Oleg Kalnichevski</a>
  * 
  * @since 2.0
  */
@@ -79,14 +80,71 @@ public interface HttpConnectionManager {
      * timeout expires
      * 
      * @see HttpConnection#setHttpConnectionManager(HttpConnectionManager)
+     * 
+     * @deprecated Use #getConnectionWithTimeout(HostConfiguration, long) 
      */
     HttpConnection getConnection(HostConfiguration hostConfiguration, long timeout)
         throws HttpException;
 
+    /**
+     * Gets an HttpConnection for a given host configuration. If a connection is
+     * not available, this method will block for at most the specified number of
+     * milliseconds or until a connection becomes available.
+     *
+     * The connection manager should be registered with any HttpConnection that
+     * is created.
+     *
+     * @param hostConfiguration the host configuration to use to configure the
+     * connection
+     * @param timeout - the time (in milliseconds) to wait for a connection to
+     * become available, 0 to specify an infinite timeout
+     * 
+     * @return an HttpConnection for the given configuraiton
+     * 
+     * @throws ConnectionPoolTimeoutException if no connection becomes available before the
+     * timeout expires
+     * 
+     * @see HttpConnection#setHttpConnectionManager(HttpConnectionManager)
+     * 
+     * @since 3.0
+     */
+    HttpConnection getConnectionWithTimeout(HostConfiguration hostConfiguration, long timeout)
+        throws ConnectionPoolTimeoutException;
+
     /**
      * Releases the given HttpConnection for use by other requests.
      *
      * @param conn - The HttpConnection to make available.
      */
     void releaseConnection(HttpConnection conn);
+
+    /**
+     * Closes connections that have been idle for at least the given amount of time.  Only
+     * connections that are currently owned, not checked out, are subject to idle timeouts.
+     * 
+     * @param idleTimeout the minimum idle time, in milliseconds, for connections to be closed
+     * 
+     * @since 3.0
+     */
+    void closeIdleConnections(long idleTimeout);
+    
+    /**
+     * Returns {@link HttpConnectionManagerParams parameters} associated 
+     * with this connection manager.
+     * 
+     * @since 3.0
+     * 
+     * @see HttpConnectionManagerParams
+     */
+    HttpConnectionManagerParams getParams();
+
+    /**
+     * Assigns {@link HttpConnectionManagerParams parameters} for this 
+     * connection manager.
+     * 
+     * @since 3.0
+     * 
+     * @see HttpConnectionManagerParams
+     */
+    void setParams(final HttpConnectionManagerParams params);
 }
