@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/HttpMethod.java,v 1.23.2.5 2004/06/13 20:24:48 olegk Exp $
- * $Revision: 1.23.2.5 $
- * $Date: 2004/06/13 20:24:48 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/HttpMethod.java,v 1.43 2004/10/07 16:14:15 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  *
  * ====================================================================
  *
@@ -25,8 +25,6 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient;
@@ -34,6 +32,9 @@ package org.apache.commons.httpclient;
 import java.io.IOException;
 import java.io.InputStream;
 
+import org.apache.commons.httpclient.auth.AuthState;
+import org.apache.commons.httpclient.params.HttpMethodParams;
+
 /**
  * <p>
  * HttpMethod interface represents a request to be sent via a 
@@ -45,7 +46,7 @@ import java.io.InputStream;
  * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
  * @author <a href="mailto:oleg@ural.ru">Oleg Kalnichevski</a>
  *
- * @version $Revision: 1.23.2.5 $ $Date: 2004/06/13 20:24:48 $
+ * @version $Revision: 155418 $ $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  * 
  * @since 1.0
  */
@@ -64,7 +65,9 @@ public interface HttpMethod {
     /**
      * Gets the host configuration for this method.  The configuration specifies
      * the server, port, protocol, and proxy server via which this method will
-     * send its HTTP request. 
+     * send its HTTP request.
+     * 
+     * @deprecated no longer applicable 
      * 
      * @return the HostConfiguration or <code>null</code> if none is set
      */
@@ -101,6 +104,17 @@ public interface HttpMethod {
      */
     URI getURI() throws URIException;
 
+    /**
+     * Sets the URI for this method. 
+     * 
+     * @param uri URI to be set 
+     * 
+     * @throws URIException if a URI cannot be set
+     * 
+     * @since 3.0
+     */
+    void setURI(URI uri) throws URIException;
+
     /**
      * Defines how strictly the method follows the HTTP protocol specification.  
      * (See RFC 2616 and other relevant RFCs.) In the strict mode the method precisely
@@ -110,6 +124,9 @@ public interface HttpMethod {
      * 
      * @param strictMode <tt>true</tt> for strict mode, <tt>false</tt> otherwise
      * 
+     * @deprecated Use {@link org.apache.commons.httpclient.params.HttpParams#setParameter(String, Object)} 
+     * to exercise a more granular control over HTTP protocol strictness.
+     * 
      * @see #isStrictMode()
      */
     void setStrictMode(boolean strictMode);
@@ -119,30 +136,33 @@ public interface HttpMethod {
      *
      * @return <tt>true</tt> if strict mode is enabled, <tt>false</tt> otherwise
      * 
+     * @deprecated Use {@link org.apache.commons.httpclient.params.HttpParams#setParameter(String, Object)} 
+     * to exercise a more granular control over HTTP protocol strictness.
+     * 
      * @see #setStrictMode(boolean)
      */
     boolean isStrictMode();
      
     /**
-     * Sets the specified request header, overwriting any previous value.
+     * Sets the specified request header, overwriting any
+     * previous value.
      * Note that header-name matching is case insensitive.
-     * 
      * @param headerName the header's name
      * @param headerValue the header's value
      *
-     * @see #setRequestHeader(Header) 
+     * @see #setRequestHeader(Header)
      * @see #getRequestHeader(String)
      * @see #removeRequestHeader(String)
      */
     void setRequestHeader(String headerName, String headerValue);
 
     /**
-     * Sets the specified request header, overwriting any previous value.
+     * Sets the specified request header, overwriting any
+     * previous value.
      * Note that header-name matching is case insensitive.
-     * 
      * @param header the header to be set
      *
-     * @see #setRequestHeader(String,String) 
+     * @see #setRequestHeader(String,String)
      * @see #getRequestHeader(String)
      * @see #removeRequestHeader(String)
      */
@@ -153,7 +173,6 @@ public interface HttpMethod {
      * If the same header is added multiple times, perhaps with different values,
      * multiple instances of that header will be sent in the HTTP request.
      * Note that header-name matching is case insensitive.
-     * 
      * @param headerName the header's name
      * @param headerValue the header's value
      * 
@@ -168,7 +187,6 @@ public interface HttpMethod {
      * If the same header is added multiple times, perhaps with different values,
      * multiple instances of that header will be sent in the HTTP request.
      * Note that header-name matching is case insensitive.
-     * 
      * @param header the header
      * 
      * @see #addRequestHeader(String,String)
@@ -180,9 +198,8 @@ public interface HttpMethod {
     /**
      * Gets the request header with the given name.
      * If there are multiple headers with the same name,
-     * there values will be combined with the ',' separator as specified by RFC2616.     
+     * there values will be combined with the ',' separator as specified by RFC2616.
      * Note that header-name matching is case insensitive.
-     * 
      * @param headerName the header name
      * @return the header
      */
@@ -191,11 +208,19 @@ public interface HttpMethod {
     /**
      * Removes all request headers with the given name.
      * Note that header-name matching is case insensitive.
-     * 
      * @param headerName the header name
      */
     void removeRequestHeader(String headerName);
 
+    /**
+     * Removes the given request header.
+     * 
+     * @param header the header
+     * 
+     * @since 3.0
+     */
+    void removeRequestHeader(Header header);
+
     /**
      * Returns <tt>true</tt> if the HTTP method should automatically follow HTTP redirects 
      * (status code 302, etc.), <tt>false</tt> otherwise.
@@ -210,7 +235,7 @@ public interface HttpMethod {
      * (status code 302, etc.)
      * 
      * @param followRedirects <tt>true</tt> if the method will automatically follow redirects,
-     * <tt>false</tt> otherwise
+     * <tt>false</tt> otherwise.
      */
     void setFollowRedirects(boolean followRedirects);
 
@@ -264,6 +289,16 @@ public interface HttpMethod {
      */
     Header[] getRequestHeaders();
 
+    /**
+     * Returns the request headers with the given name. Note that header-name matching is
+     * case insensitive.
+     * @param headerName the name of the headers to be returned.
+     * @return an array of zero or more headers
+     * 
+     * @since 3.0
+     */
+    Header[] getRequestHeaders(String headerName);
+
     // ---------------------------------------------------------------- Queries
 
     /**
@@ -310,6 +345,16 @@ public interface HttpMethod {
      */
     Header getResponseHeader(String headerName);
 
+    /**
+     * Returns the response headers with the given name. Note that header-name matching is
+     * case insensitive.
+     * @param headerName the name of the headers to be returned.
+     * @return an array of zero or more headers
+     * 
+     * @since 3.0
+     */
+    Header[] getResponseHeaders(String headerName);
+
     /**
      * Returns the response footers from the most recent execution of this request.
      * 
@@ -336,8 +381,10 @@ public interface HttpMethod {
      * 
      * @return The response body, or <code>null</code> if the
      *         body is not available.
+     * 
+     * @throws IOException if an I/O (transport) problem occurs
      */
-    byte[] getResponseBody();
+    byte[] getResponseBody() throws IOException;
 
     /**
      * Returns the response body of the HTTP method, if any, as a {@link String}. 
@@ -351,8 +398,10 @@ public interface HttpMethod {
      *
      * @return The response body converted to a <code>String</code>, or <code>null</code>
      *         if the body is not available.
+     * 
+     * @throws IOException if an I/O (transport) problem occurs
      */
-    String getResponseBodyAsString();
+    String getResponseBodyAsString() throws IOException;
 
     /**
      * Returns the response body of the HTTP method, if any, as an InputStream.
@@ -395,6 +444,15 @@ public interface HttpMethod {
     int execute(HttpState state, HttpConnection connection) 
         throws HttpException, IOException;
 
+    /**
+     * Aborts the execution of the HTTP method.
+     * 
+     * @see #execute(HttpState, HttpConnection)
+     * 
+     * @since 3.0
+     */
+    void abort();
+
     /**
      * Recycles the HTTP method so that it can be used again.
      * Note that all of the instance variables will be reset
@@ -468,4 +526,51 @@ public interface HttpMethod {
      */
     void setDoAuthentication(boolean doAuthentication);
 
+
+    /**
+     * Returns {@link HttpMethodParams HTTP protocol parameters} associated with this method.
+     * 
+     * @since 3.0
+     * 
+     * @see HttpMethodParams
+     */
+    public HttpMethodParams getParams();
+
+    /**
+     * Assigns {@link HttpMethodParams HTTP protocol parameters} for this method.
+     * 
+     * @since 3.0
+     * 
+     * @see HttpMethodParams
+     */
+    public void setParams(final HttpMethodParams params);
+
+    /**
+     * Returns the target host {@link AuthState authentication state}
+     * 
+     * @return host authentication state
+     * 
+     * @since 3.0
+     */
+    public AuthState getHostAuthState();
+
+    /**
+     * Returns the proxy {@link AuthState authentication state}
+     * 
+     * @return host authentication state
+     * 
+     * @since 3.0
+     */
+    public AuthState getProxyAuthState();
+
+    /**
+     * Returns <tt>true</tt> if the HTTP has been transmitted to the target
+     * server in its entirety, <tt>false</tt> otherwise. This flag can be useful 
+     * for recovery logic. If the request has not been transmitted in its entirety,
+     * it is safe to retry the failed method.
+     * 
+     * @return <tt>true</tt> if the request has been sent, <tt>false</tt> otherwise
+     */
+    boolean isRequestSent();
+
 }
