@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/HttpState.java,v 1.22.2.5 2004/09/22 23:36:58 mbecke Exp $
- * $Revision: 1.22.2.5 $
- * $Date: 2004/09/22 23:36:58 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/HttpState.java,v 1.38 2004/12/20 11:50:54 olegk Exp $
+ * $Revision: 377704 $
+ * $Date: 2006-02-14 06:48:29 -0500 (Tue, 14 Feb 2006) $
  *
  * ====================================================================
  *
@@ -25,8 +25,6 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient;
@@ -39,7 +37,7 @@ import java.util.List;
 import java.util.Iterator;
 import org.apache.commons.httpclient.cookie.CookieSpec;
 import org.apache.commons.httpclient.cookie.CookiePolicy;
-import org.apache.commons.httpclient.auth.HttpAuthRealm; 
+import org.apache.commons.httpclient.auth.AuthScope; 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -49,11 +47,6 @@ import org.apache.commons.logging.LogFactory;
  * to request, such as {@link Cookie cookies} and authentication
  * {@link Credentials credentials}.
  * </p>
- * <p>
- * Preemptive authentication can be turned on by using the property value of
- * #PREEMPTIVE_PROPERTY.  If left unspecified, it has the default value of
- * #PREEMPTIVE_DEFAULT.  This configurable behaviour conforms to rcf2617:
- * </p>
  * 
  * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
  * @author Rodney Waldhoff
@@ -64,30 +57,13 @@ import org.apache.commons.logging.LogFactory;
  * @author <a href="mailto:mbowler@GargoyleSoftware.com">Mike Bowler</a>
  * @author <a href="mailto:adrian@intencha.com">Adrian Sutton</a>
  * 
- * @version $Revision: 1.22.2.5 $ $Date: 2004/09/22 23:36:58 $
+ * @version $Revision: 377704 $ $Date: 2006-02-14 06:48:29 -0500 (Tue, 14 Feb 2006) $
  * 
  */
 public class HttpState {
 
     // ----------------------------------------------------- Instance Variables
 
-    /**
-     * Whether authentication should be attempted preemptively.
-     */
-    private boolean preemptive;
-
-    /**
-     * The boolean system property name to turn on preemptive authentication.
-     * @deprecated This field and feature will be removed following HttpClient 3.0.
-     */
-    public static final String PREEMPTIVE_PROPERTY = "httpclient.authentication.preemptive";
-
-    /**
-     * The default value for {@link #PREEMPTIVE_PROPERTY}.
-     * @deprecated This field and feature will be removed following HttpClient 3.0.
-     */
-    public static final String PREEMPTIVE_DEFAULT = "false";
-
     /**
      * Map of {@link Credentials credentials} by realm that this 
      * HTTP state contains.
@@ -100,26 +76,28 @@ public class HttpState {
      */
     private HashMap proxyCred = new HashMap();
 
-    /**
-     * The default authentication realm.
-     */
-    public static final HttpAuthRealm DEFAULT_AUTH_REALM = new HttpAuthRealm(null, null); 
-
     /**
      * Array of {@link Cookie cookies} that this HTTP state contains.
      */
     private ArrayList cookies = new ArrayList();
+
+    private boolean preemptive = false;
+
+    private int cookiePolicy = -1;
+        // -------------------------------------------------------- Class Variables
+
     /**
-     * Cookie policy that applies to this HTTP state. Default is 
-     * {@link CookiePolicy#RFC2109}
+     * The boolean system property name to turn on preemptive authentication.
+     * @deprecated This field and feature will be removed following HttpClient 3.0.
      */
-    private int cookiePolicy = CookiePolicy.RFC2109;
-
-    /** The current connection manager */
-    private HttpConnectionManager httpConnectionManager;
-
-    // -------------------------------------------------------- Class Variables
+    public static final String PREEMPTIVE_PROPERTY = "httpclient.authentication.preemptive";
 
+    /**
+     * The default value for {@link #PREEMPTIVE_PROPERTY}.
+     * @deprecated This field and feature will be removed following HttpClient 3.0.
+     */
+    public static final String PREEMPTIVE_DEFAULT = "false";
+    
     /** Log object for this class. */
     private static final Log LOG = LogFactory.getLog(HttpState.class);
 
@@ -127,30 +105,7 @@ public class HttpState {
      * Default constructor.
      */
     public HttpState() {
-        
         super();
-        
-        this.cookiePolicy = CookiePolicy.getDefaultPolicy();
-
-        // check the preemptive policy
-        String preemptiveDefault = null;
-        try {
-            preemptiveDefault = System.getProperty(PREEMPTIVE_PROPERTY); 
-        } catch (SecurityException ignore) {
-        }
-        if (preemptiveDefault == null) {
-            preemptiveDefault = PREEMPTIVE_DEFAULT; 
-        }
-        preemptiveDefault = preemptiveDefault.trim().toLowerCase();
-
-        if (!(preemptiveDefault.equals("true")
-                    || preemptiveDefault.equals("false"))) { // property problem
-            LOG.warn("Configuration property " + PREEMPTIVE_PROPERTY
-                     + " must be either true or false.  Using default: "
-                     + PREEMPTIVE_DEFAULT);
-            preemptiveDefault = PREEMPTIVE_DEFAULT;
-        }
-        this.preemptive = ("true".equals(preemptiveDefault));
     }
 
     // ------------------------------------------------------------- Properties
@@ -192,6 +147,7 @@ public class HttpState {
      * 
      * @see #addCookie(Cookie)
      * 
+     * 
      */
     public synchronized void addCookies(Cookie[] cookies) {
         LOG.trace("enter HttpState.addCookies(Cookie[])");
@@ -209,7 +165,7 @@ public class HttpState {
      * 
      * @return an array of {@link Cookie cookies}.
      * 
-     * @see #getCookies(String, int, String, boolean, java.util.Date)
+     * @see #getCookies(String, int, String, boolean)
      * 
      */
     public synchronized Cookie[] getCookies() {
@@ -225,7 +181,6 @@ public class HttpState {
      * @param port the request port
      * @param path the request path
      * @param secure <code>true</code> when using HTTPS
-     * @param now the {@link Date date} by which expiration is determined
      * 
      * @return an array of {@link Cookie cookies}.
      * 
@@ -233,32 +188,6 @@ public class HttpState {
      * 
      * @deprecated use CookieSpec#match(String, int, String, boolean, Cookie)
      */
-    public synchronized Cookie[] getCookies(
-        String domain, 
-        int port, 
-        String path, 
-        boolean secure, 
-        Date now
-    ) {
-        return getCookies(domain, port, path, secure);
-    }
-
-
-    /**
-     * Returns an array of {@link Cookie cookies} in this HTTP
-     * state that match the given request parameters.
-     * 
-     * @param domain the request domain
-     * @param port the request port
-     * @param path the request path
-     * @param secure <code>true</code> when using HTTPS
-     * 
-     * @return an array of {@link Cookie cookies}.
-     * 
-     * @see #getCookies()
-     * 
-     * @deprecated use {@link CookieSpec#match(String, int, String, boolean, Cookie)}
-     */
     public synchronized Cookie[] getCookies(
         String domain, 
         int port, 
@@ -321,6 +250,10 @@ public class HttpState {
      * HTTP state.
      * 
      * @return The {@link CookiePolicy cookie policy}.
+     * 
+     * @deprecated Use 
+     *  {@link org.apache.commons.httpclient.params.HttpMethodParams#getCookiePolicy()},
+     *  {@link HttpMethod#getParams()}.     
      */
     
     public int getCookiePolicy() {
@@ -334,6 +267,10 @@ public class HttpState {
      * 
      * @param value <tt>true</tt> if preemptive authentication should be 
      * attempted, <tt>false</tt> otherwise. 
+     * 
+     * @deprecated Use 
+     * {@link org.apache.commons.httpclient.params.HttpClientParams#setAuthenticationPreemptive(boolean)}, 
+     * {@link HttpClient#getParams()}.
      */
     
     public void setAuthenticationPreemptive(boolean value) {
@@ -346,6 +283,10 @@ public class HttpState {
      * attempted, <tt>false</tt> otherwise.
      * 
      * @return boolean flag.
+     * 
+     * @deprecated Use 
+     * {@link org.apache.commons.httpclient.params.HttpClientParams#isAuthenticationPreemptive()}, 
+     * {@link HttpClient#getParams()}.
      */
     
     public boolean isAuthenticationPreemptive() {
@@ -361,35 +302,16 @@ public class HttpState {
      * {@link CookiePolicy#RFC2109}.
      * 
      * @param policy new {@link CookiePolicy cookie policy}
+     * 
+     * @deprecated 
+     *  Use {@link org.apache.commons.httpclient.params.HttpMethodParams#setCookiePolicy(String)},
+     *  {@link HttpMethod#getParams()}.     
      */
     
     public void setCookiePolicy(int policy) {
         this.cookiePolicy = policy;
     }
 
-    /**
-     * Sets the {@link Credentials credentials} for the given authentication 
-     * realm. The <code>null</code> realm signifies default credentials, which 
-     * should be used when no {@link Credentials credentials} have been explictly 
-     * supplied for the given challenging realm. Any previous credentials for
-     * the given realm will be overwritten.
-     *
-     * @deprecated This method does not distinguish between realms with the
-     * same name on different hosts.  
-     * Use {@link #setCredentials(String, String, Credentials)} instead.
-     * 
-     * @param realm the authentication realm
-     * @param credentials the authentication credentials for the given realm
-     * 
-     * @see #getCredentials(String, String)
-     * @see #setProxyCredentials(String, String, Credentials)
-     */
-    
-    public synchronized void setCredentials(String realm, Credentials credentials) {
-        LOG.trace("enter HttpState.setCredentials(String, Credentials)");
-        setCredentials(realm, null, credentials);
-    }
-    
     /** 
      * Sets the {@link Credentials credentials} for the given authentication 
      * realm on the given host. The <code>null</code> realm signifies default 
@@ -407,51 +329,70 @@ public class HttpState {
      * 
      * @see #getCredentials(String, String)
      * @see #setProxyCredentials(String, String, Credentials) 
+     * 
+     * @deprecated use #setCredentials(AuthScope, Credentials)
      */
     
     public synchronized void setCredentials(String realm, String host, Credentials credentials) {
-        LOG.trace(
-            "enter HttpState.setCredentials(String realm, String host, Credentials credentials)");
-        credMap.put(new HttpAuthRealm(host, realm), credentials);
+        LOG.trace("enter HttpState.setCredentials(String, String, Credentials)");
+        credMap.put(new AuthScope(host, AuthScope.ANY_PORT, realm, AuthScope.ANY_SCHEME), credentials);
     }
 
+    /** 
+     * Sets the {@link Credentials credentials} for the given authentication 
+     * scope. Any previous credentials for the given scope will be overwritten.
+     * 
+     * @param authscope the {@link AuthScope authentication scope}
+     * @param credentials the authentication {@link Credentials credentials} 
+     * for the given scope.
+     * 
+     * @see #getCredentials(AuthScope)
+     * @see #setProxyCredentials(AuthScope, Credentials) 
+     * 
+     * @since 3.0
+     */
+    public synchronized void setCredentials(final AuthScope authscope, final Credentials credentials) {
+        if (authscope == null) {
+            throw new IllegalArgumentException("Authentication scope may not be null");
+        }
+        LOG.trace("enter HttpState.setCredentials(AuthScope, Credentials)");
+        credMap.put(authscope, credentials);
+    }
 
     /**
-     * Find matching {@link Credentials credentials} for the given authentication realm and host.
-     *
-     * If the <i>realm</i> exists on <i>host</i>, return the coresponding credentials.
-     * If the <i>host</i> exists with a <tt>null</tt> <i>realm</i>, return the corresponding
-     * credentials.
-     * If the <i>realm</i> exists with a <tt>null</tt> <i>host</i>, return the
-     * corresponding credentials.  If the <i>realm</i> does not exist, return
-     * the default Credentials.  If there are no default credentials, return
-     * <code>null</code>.
+     * Find matching {@link Credentials credentials} for the given authentication scope.
      *
      * @param map the credentials hash map
-     * @param realm the authentication realm
-     * @param host the host the realm is on
+     * @param token the {@link AuthScope authentication scope}
      * @return the credentials 
      * 
      */
-    private static Credentials matchCredentials(HashMap map, String realm, String host) {
-        HttpAuthRealm entry = new HttpAuthRealm(host, realm);
-        Credentials creds = (Credentials) map.get(entry);
-        if (creds == null && host != null && realm != null) {
-            entry = new HttpAuthRealm(host, null);
-            creds = (Credentials) map.get(entry);
-            if (creds == null) {
-                entry = new HttpAuthRealm(null, realm);
-                creds = (Credentials) map.get(entry);
-            }
-        }
+    private static Credentials matchCredentials(final HashMap map, final AuthScope authscope) {
+        // see if we get a direct hit
+        Credentials creds = (Credentials)map.get(authscope);
         if (creds == null) {
-            creds = (Credentials) map.get(DEFAULT_AUTH_REALM);
+            // Nope.
+            // Do a full scan
+            int bestMatchFactor  = -1;
+            AuthScope bestMatch  = null;
+            Iterator items = map.keySet().iterator();
+            while (items.hasNext()) {
+                AuthScope current = (AuthScope)items.next();
+                int factor = authscope.match(current);
+                if (factor > bestMatchFactor) {
+                    bestMatchFactor = factor;
+                    bestMatch = current;
+                }
+            }
+            if (bestMatch != null) {
+                creds = (Credentials)map.get(bestMatch);
+            }
         }
         return creds;
     }
     
     /**
-     * Get the {@link Credentials credentials} for the given authentication realm on the 
+     * Get the {@link Credentials credentials} for the given authentication scope on the 
      * given host.
      *
      * If the <i>realm</i> exists on <i>host</i>, return the coresponding credentials.
@@ -467,57 +408,34 @@ public class HttpState {
      * @return the credentials 
      * 
      * @see #setCredentials(String, String, Credentials)
+     * 
+     * @deprecated use #getCredentials(AuthScope)
      */
     
     public synchronized Credentials getCredentials(String realm, String host) {
         LOG.trace("enter HttpState.getCredentials(String, String");
-        return matchCredentials(this.credMap, realm, host);
+        return matchCredentials(this.credMap, 
+            new AuthScope(host, AuthScope.ANY_PORT, realm, AuthScope.ANY_SCHEME));
     }
 
     /**
-     * Get the {@link Credentials credentials} for the given authentication realm.
+     * Get the {@link Credentials credentials} for the given authentication scope.
      *
-     * @deprecated This method does not distinguish between realms on different
-     * servers with the same name.  Use {@link #getCredentials(String, String)}
-     * instead.
-     * 
-     * @param realm the authentication realm
+     * @param authscope the {@link AuthScope authentication scope}
      * @return the credentials 
      * 
-     * @see #setCredentials(String, String, Credentials)
+     * @see #setCredentials(AuthScope, Credentials)
      * 
+     * @since 3.0
      */
-    
-    public synchronized Credentials getCredentials(String realm) {
-        LOG.trace("enter HttpState.getCredentials(String)");
-
-        return getCredentials(realm, null);
+    public synchronized Credentials getCredentials(final AuthScope authscope) {
+        if (authscope == null) {
+            throw new IllegalArgumentException("Authentication scope may not be null");
+        }
+        LOG.trace("enter HttpState.getCredentials(AuthScope)");
+        return matchCredentials(this.credMap, authscope);
     }
 
-    /**
-     * Sets the {@link Credentials credentials} for the given proxy authentication 
-     * realm. The <code>null</code> realm signifies default credentials, which 
-     * should be used when no {@link Credentials credentials} have been explictly 
-     * supplied for the given challenging proxy realm. Any previous credentials for
-     * the given realm will be overwritten.
-     * 
-     * @deprecated This method does not differentiate between realms with
-     * the same name on different servers.  Use
-     * {@link #setProxyCredentials(String, String, Credentials)} instead.
-     * 
-     * @param realm the authentication realm
-     * @param credentials the authentication credentials for the given realm
-     * 
-     * @see #getProxyCredentials(String)
-     * @see #setCredentials(String, Credentials)
-     * 
-     */
-    
-    public synchronized void setProxyCredentials(String realm, Credentials credentials) {
-        LOG.trace("enter HttpState.setProxyCredentials(String, credentials)");
-        setProxyCredentials(realm, null, credentials);
-    }
-    
     /**
      * Sets the {@link Credentials credentials} for the given proxy authentication 
      * realm on the given proxy host. The <code>null</code> proxy realm signifies 
@@ -528,14 +446,15 @@ public class HttpState {
      * have been explictly supplied for the challenging proxy host. Any previous 
      * credentials for the given proxy realm on the given proxy host will be 
      * overwritten.
-     * 
+     *
      * @param realm the authentication realm
      * @param proxyHost the proxy host
      * @param credentials the authentication credentials for the given realm
      * 
-     * @see #getProxyCredentials(String)
-     * @see #setCredentials(String, Credentials)
+     * @see #getProxyCredentials(AuthScope)
+     * @see #setCredentials(AuthScope, Credentials)
      * 
+     * @deprecated use #setProxyCredentials(AuthScope, Credentials)
      */
     public synchronized void setProxyCredentials(
         String realm, 
@@ -543,33 +462,35 @@ public class HttpState {
         Credentials credentials
     ) {
         LOG.trace("enter HttpState.setProxyCredentials(String, String, Credentials");
-        proxyCred.put(new HttpAuthRealm(proxyHost, realm), credentials);
+        proxyCred.put(new AuthScope(proxyHost, AuthScope.ANY_PORT, realm, AuthScope.ANY_SCHEME), credentials);
+    }
+
+    /** 
+     * Sets the {@link Credentials proxy credentials} for the given authentication 
+     * realm. Any previous credentials for the given realm will be overwritten.
+     * 
+     * @param authscope the {@link AuthScope authentication scope}
+     * @param credentials the authentication {@link Credentials credentials} 
+     * for the given realm.
+     * 
+     * @see #getProxyCredentials(AuthScope)
+     * @see #setCredentials(AuthScope, Credentials) 
+     * 
+     * @since 3.0
+     */
+    public synchronized void setProxyCredentials(final AuthScope authscope, 
+        final Credentials credentials)
+    {
+        if (authscope == null) {
+            throw new IllegalArgumentException("Authentication scope may not be null");
+        }
+        LOG.trace("enter HttpState.setProxyCredentials(AuthScope, Credentials)");
+        proxyCred.put(authscope, credentials);
     }
 
-    /**
-     * Get the {@link Credentials credentials} for the given 
-     * proxy authentication realm.
-     *
-     * If the <i>realm</i> exists, return the coresponding credentials.  If the 
-     * <i>realm</i> does not exist, return the default Credentials.  If there is 
-     * no default credentials, return <code>null</code>.
-     * 
-     * @deprecated This method does not distinguish between realms on different hosts.
-     * Use {@link #getProxyCredentials(String, String)} instead.
-     *
-     * @param realm the authentication realm
-     * @return the credentials 
-     * @see #setProxyCredentials(String, String, Credentials)
-     */
-    
-    public synchronized Credentials getProxyCredentials(String realm) {
-        LOG.trace("enter HttpState.getProxyCredentials(String)");
-        return getProxyCredentials(realm, null);
-    }
-    
     /**
      * Get the {@link Credentials credentials} for the proxy host with the given 
-     * authentication realm.
+     * authentication scope.
      *
      * If the <i>realm</i> exists on <i>host</i>, return the coresponding credentials.
      * If the <i>host</i> exists with a <tt>null</tt> <i>realm</i>, return the corresponding
@@ -583,12 +504,33 @@ public class HttpState {
      * @param proxyHost the proxy host the realm is on
      * @return the credentials 
      * @see #setProxyCredentials(String, String, Credentials)
+     * 
+     * @deprecated use #getProxyCredentials(AuthScope)
      */
     public synchronized Credentials getProxyCredentials(String realm, String proxyHost) {
        LOG.trace("enter HttpState.getCredentials(String, String");
-        return matchCredentials(this.proxyCred, realm, proxyHost);
+        return matchCredentials(this.proxyCred, 
+            new AuthScope(proxyHost, AuthScope.ANY_PORT, realm, AuthScope.ANY_SCHEME));
     }
     
+    /**
+     * Get the {@link Credentials proxy credentials} for the given authentication scope.
+     *
+     * @param authscope the {@link AuthScope authentication scope}
+     * @return the credentials 
+     * 
+     * @see #setProxyCredentials(AuthScope, Credentials)
+     * 
+     * @since 3.0
+     */
+    public synchronized Credentials getProxyCredentials(final AuthScope authscope) {
+        if (authscope == null) {
+            throw new IllegalArgumentException("Authentication scope may not be null");
+        }
+        LOG.trace("enter HttpState.getProxyCredentials(AuthScope)");
+        return matchCredentials(this.proxyCred, authscope);
+    }
+
     /**
      * Returns a string representation of this HTTP state.
      * 
@@ -600,10 +542,10 @@ public class HttpState {
         StringBuffer sbResult = new StringBuffer();
 
         sbResult.append("[");
-        sbResult.append(getProxyCredentialsStringRepresentation(proxyCred));
-        sbResult.append(" | ");
         sbResult.append(getCredentialsStringRepresentation(proxyCred));
         sbResult.append(" | ");
+        sbResult.append(getCredentialsStringRepresentation(credMap));
+        sbResult.append(" | ");
         sbResult.append(getCookiesStringRepresentation(cookies));
         sbResult.append("]");
 
@@ -612,27 +554,6 @@ public class HttpState {
         return strResult;
     }
     
-    /**
-     * Returns a string representation of the proxy credentials
-     * @param proxyCredMap The proxy credentials
-     * @return The string representation.
-     */
-    private static String getProxyCredentialsStringRepresentation(final Map proxyCredMap) {
-        StringBuffer sbResult = new StringBuffer();
-        Iterator iter = proxyCredMap.keySet().iterator();
-        while (iter.hasNext()) {
-            Object key = iter.next();
-            Credentials cred = (Credentials) proxyCredMap.get(key);
-            if (sbResult.length() > 0) {
-                sbResult.append(", ");
-            }
-            sbResult.append(key);
-            sbResult.append("#");
-            sbResult.append(cred.toString());
-        }
-        return sbResult.toString();
-    }
-    
     /**
      * Returns a string representation of the credentials.
      * @param credMap The credentials.
@@ -655,7 +576,7 @@ public class HttpState {
     }
     
     /**
-     * Return a string representation of the cookies.
+     * Returns a string representation of the cookies.
      * @param cookies The cookies
      * @return The string representation.
      */
@@ -673,30 +594,32 @@ public class HttpState {
     }
     
     /**
-     * Returns the httpConnectionManager.
-     * @return HttpConnectionManager
-     * 
-     * @deprecated Connection manager is controlled by the HttpClient class.
-     * Use {@link HttpClient#getHttpConnectionManager()} instead.
-     * 
-     * @since 2.0
+     * Clears all credentials.
      */
-    public synchronized HttpConnectionManager getHttpConnectionManager() {
-        return httpConnectionManager;
+    public void clearCredentials() {
+        this.credMap.clear();
     }
-
+    
     /**
-     * Sets the httpConnectionManager.
-     * @param httpConnectionManager The httpConnectionManager to set
-     * 
-     * @deprecated Connection manager is controlled by the HttpClient class.
-     * Use {@link HttpClient#setHttpConnectionManager(HttpConnectionManager)} instead.
-     *
-     * @since 2.0
+     * Clears all proxy credentials.
      */
-    public synchronized void setHttpConnectionManager(
-        HttpConnectionManager httpConnectionManager
-    ) {
-        this.httpConnectionManager = httpConnectionManager;
+    public void clearProxyCredentials() {
+        this.proxyCred.clear();
+    }
+    
+    /**
+     * Clears all cookies.
+     */
+    public synchronized void clearCookies() {
+        this.cookies.clear();
+    }
+    
+    /**
+     * Clears the state information (all cookies, credentials and proxy credentials).
+     */
+    public void clear() {
+        clearCookies();
+        clearCredentials();
+        clearProxyCredentials();
     }
 }
