@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/HttpURL.java,v 1.12.2.5 2004/09/30 17:26:27 oglueck Exp $
- * $Revision: 1.12.2.5 $
- * $Date: 2004/09/30 17:26:27 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/HttpURL.java,v 1.18 2004/09/30 17:26:41 oglueck Exp $
+ * $Revision: 358634 $
+ * $Date: 2005-12-22 16:23:15 -0500 (Thu, 22 Dec 2005) $
  *
  * ====================================================================
  *
@@ -25,8 +25,6 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient;
@@ -830,5 +828,51 @@ public class HttpURL extends URI {
         }
     }
 
+    /**
+     * Once it's parsed successfully, set this URI.
+     *
+     * @see #getRawURI
+     */
+    protected void setURI() {
+        // set _uri
+        StringBuffer buf = new StringBuffer();
+        // ^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?
+        if (_scheme != null) {
+            buf.append(_scheme);
+            buf.append(':');
+        }
+        if (_is_net_path) {
+            buf.append("//");
+            if (_authority != null) { // has_authority
+                if (_userinfo != null) { // by default, remove userinfo part
+                    if (_host != null) {
+                        buf.append(_host);
+                        if (_port != -1) {
+                            buf.append(':');
+                            buf.append(_port);
+                        }
+                    }
+                } else {
+                    buf.append(_authority);
+                }
+            }
+        }
+        if (_opaque != null && _is_opaque_part) {
+            buf.append(_opaque);
+        } else if (_path != null) {
+            // _is_hier_part or _is_relativeURI
+            if (_path.length != 0) {
+                buf.append(_path);
+            }
+        }
+        if (_query != null) { // has_query
+            buf.append('?');
+            buf.append(_query);
+        }
+        // ignore the fragment identifier
+        _uri = buf.toString().toCharArray();
+        hash = 0;
+    }
+    
 }
 
