@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/cookie/CookieSpecBase.java,v 1.16.2.3 2004/02/22 18:21:15 olegk Exp $
- * $Revision: 1.16.2.3 $
- * $Date: 2004/02/22 18:21:15 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/cookie/CookieSpecBase.java,v 1.28 2004/11/06 19:15:42 mbecke Exp $
+ * $Revision: 190485 $
+ * $Date: 2005-06-13 15:04:56 -0400 (Mon, 13 Jun 2005) $
  *
  * ====================================================================
  *
@@ -25,12 +25,11 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */ 
 
 package org.apache.commons.httpclient.cookie;
 
+import java.util.Collection;
 import java.util.Date;
 import java.util.LinkedList;
 import java.util.List;
@@ -38,10 +37,9 @@ import java.util.List;
 import org.apache.commons.httpclient.Cookie;
 import org.apache.commons.httpclient.Header;
 import org.apache.commons.httpclient.HeaderElement;
-import org.apache.commons.httpclient.HttpException;
 import org.apache.commons.httpclient.NameValuePair;
 import org.apache.commons.httpclient.util.DateParseException;
-import org.apache.commons.httpclient.util.DateParser;
+import org.apache.commons.httpclient.util.DateUtil;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -67,6 +65,9 @@ public class CookieSpecBase implements CookieSpec {
     /** Log object */
     protected static final Log LOG = LogFactory.getLog(CookieSpec.class);
 
+    /** Valid date patterns */
+    private Collection datepatterns = null;
+    
     /** Default constructor */
     public CookieSpecBase() {
         super();
@@ -134,14 +135,7 @@ public class CookieSpecBase implements CookieSpec {
             path = PATH_DELIM;
         }
         host = host.toLowerCase();
-    
-        HeaderElement[] headerElements = null;
-        try {
-            headerElements = HeaderElement.parse(header);
-        } catch (HttpException e) {
-            throw new MalformedCookieException(e.getMessage());
-        } 
-    
+
         String defaultPath = path;    
         int lastSlashIndex = defaultPath.lastIndexOf(PATH_DELIM);
         if (lastSlashIndex >= 0) {
@@ -151,6 +145,31 @@ public class CookieSpecBase implements CookieSpec {
             }
             defaultPath = defaultPath.substring(0, lastSlashIndex);
         }
+
+        HeaderElement[] headerElements = null;
+
+        boolean isNetscapeCookie = false; 
+        int i1 = header.toLowerCase().indexOf("expires=");
+        if (i1 != -1) {
+            i1 += "expires=".length();
+            int i2 = header.indexOf(";", i1);
+            if (i2 == -1) {
+                i2 = header.length(); 
+            }
+            try {
+                DateUtil.parseDate(header.substring(i1, i2), this.datepatterns);
+                isNetscapeCookie = true; 
+            } catch (DateParseException e) {
+                // Does not look like a valid expiry date
+            }
+        }
+        if (isNetscapeCookie) {
+            headerElements = new HeaderElement[] {
+                    new HeaderElement(header.toCharArray())
+            };
+        } else {
+            headerElements = HeaderElement.parseElements(header.toCharArray());
+        }
         
         Cookie[] cookies = new Cookie[headerElements.length];
 
@@ -303,17 +322,9 @@ public class CookieSpecBase implements CookieSpec {
                 throw new MalformedCookieException(
                     "Missing value for expires attribute");
             }
-            // trim single quotes around expiry if present
-            // see http://nagoya.apache.org/bugzilla/show_bug.cgi?id=5279
-            if (paramValue.length() > 1 
-                && paramValue.startsWith("'") 
-                && paramValue.endsWith("'")) {
-                paramValue 
-                    = paramValue.substring (1, paramValue.length() - 1);
-            }
 
             try {
-                cookie.setExpiryDate(DateParser.parseDate(paramValue));
+                cookie.setExpiryDate(DateUtil.parseDate(paramValue, this.datepatterns));
             } catch (DateParseException dpe) {
                 LOG.debug("Error parsing cookie date", dpe);
                 throw new MalformedCookieException(
@@ -329,6 +340,14 @@ public class CookieSpecBase implements CookieSpec {
     }
 
     
+    public Collection getValidDateFormats() {
+        return this.datepatterns;
+    }
+
+    public void setValidDateFormats(final Collection datepatterns) {
+        this.datepatterns = datepatterns;
+    }
+
     /**
       * Performs most common {@link Cookie} validation
       *
@@ -478,30 +497,29 @@ public class CookieSpecBase implements CookieSpec {
     }
 
     /**
-     * Performs a domain-match as described in RFC2109.
-     * @param host The host to check.
-     * @param domain The domain.
+     * Performs domain-match as implemented in common browsers.
+     * @param host The target host.
+     * @param domain The cookie domain attribute.
      * @return true if the specified host matches the given domain.
      */
-    private static boolean domainMatch(String host, String domain) {
-        boolean match = host.equals(domain) 
-            || (domain.startsWith(".") && host.endsWith(domain));
-
-        return match;
+    public boolean domainMatch(final String host, String domain) {
+        if (host.equals(domain)) {
+            return true;
+        }
+        if (!domain.startsWith(".")) {
+            domain = "." + domain;
+        }
+        return host.endsWith(domain) || host.equals(domain.substring(1));
     }
 
     /**
-     * Performs a path-match slightly smarter than a straight-forward startsWith
-     * check.
-     * @param path The path to check.
-     * @param topmostPath The path to check against.
+     * Performs path-match as implemented in common browsers.
+     * @param path The target path.
+     * @param topmostPath The cookie path attribute.
      * @return true if the paths match
      */
-    private static boolean pathMatch(
-        final String path, final String topmostPath) {
-            
+    public boolean pathMatch(final String path, final String topmostPath) {
         boolean match = path.startsWith (topmostPath);
-        
         // if there is a match and these values are not exactly the same we have
         // to make sure we're not matcing "/foobar" and "/foo"
         if (match && path.length() != topmostPath.length()) {
@@ -530,30 +548,7 @@ public class CookieSpecBase implements CookieSpec {
         LOG.trace("enter CookieSpecBase.match("
             + "String, int, String, boolean, Cookie[])");
 
-        if (host == null) {
-            throw new IllegalArgumentException(
-                "Host of origin may not be null");
-        }
-        if (host.trim().equals("")) {
-            throw new IllegalArgumentException(
-                "Host of origin may not be blank");
-        }
-        if (port < 0) {
-            throw new IllegalArgumentException("Invalid port: " + port);
-        }
-        if (path == null) {
-            throw new IllegalArgumentException(
-                "Path of origin may not be null.");
-        }
         if (cookies == null) {
-            throw new IllegalArgumentException("Cookie array may not be null");
-        }
-        if (path.trim().equals("")) {
-            path = PATH_DELIM;
-        }
-        host = host.toLowerCase();
-
-        if (cookies.length <= 0) {
             return null;
         }
         List matching = new LinkedList();
@@ -603,7 +598,7 @@ public class CookieSpecBase implements CookieSpec {
         String s = cookie.getValue();
         if (s != null) {
             buf.append(s);
-        };
+        }
         return buf.toString();
     }
 
