@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/methods/EntityEnclosingMethod.java,v 1.18.2.5 2004/06/13 20:24:49 olegk Exp $
- * $Revision: 1.18.2.5 $
- * $Date: 2004/06/13 20:24:49 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/methods/EntityEnclosingMethod.java,v 1.39 2004/07/03 14:27:03 olegk Exp $
+ * $Revision: 161963 $
+ * $Date: 2005-04-19 16:25:06 -0400 (Tue, 19 Apr 2005) $
  *
  * ====================================================================
  *
@@ -25,24 +25,22 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient.methods;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.UnsupportedEncodingException;
 
 import org.apache.commons.httpclient.ChunkedOutputStream;
-import org.apache.commons.httpclient.ContentLengthInputStream;
+import org.apache.commons.httpclient.Header;
 import org.apache.commons.httpclient.HttpConnection;
-import org.apache.commons.httpclient.HttpConstants;
 import org.apache.commons.httpclient.HttpException;
 import org.apache.commons.httpclient.HttpState;
+import org.apache.commons.httpclient.HttpVersion;
+import org.apache.commons.httpclient.ProtocolException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -54,7 +52,7 @@ import org.apache.commons.logging.LogFactory;
  * @author <a href="mailto:jsdever@apache.org">Jeff Dever</a>
  *
  * @since 2.0beta1
- * @version $Revision: 1.18.2.5 $
+ * @version $Revision: 161963 $
  */
 public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
 
@@ -63,39 +61,39 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
     /**
      * The content length will be calculated automatically. This implies
      * buffering of the content.
+     * @deprecated Use {@link InputStreamRequestEntity#CONTENT_LENGTH_AUTO}.
      */
-    public static final int CONTENT_LENGTH_AUTO = -2;
+    public static final long CONTENT_LENGTH_AUTO = InputStreamRequestEntity.CONTENT_LENGTH_AUTO;
 
     /**
      * The request will use chunked transfer encoding. Content length is not
      * calculated and the content is not buffered.<br>
+     * @deprecated Use {@link #setContentChunked(boolean)}.
      */
-    public static final int CONTENT_LENGTH_CHUNKED = -1;
+    public static final long CONTENT_LENGTH_CHUNKED = -1;
 
     /** LOG object for this class. */
     private static final Log LOG = LogFactory.getLog(EntityEnclosingMethod.class);
 
-    /** The buffered request body, if any. */
-    private byte[] buffer = null;
-
     /** The unbuffered request body, if any. */
     private InputStream requestStream = null;
 
     /** The request body as string, if any. */
     private String requestString = null;
 
-    /** for optimization purpose, the generated request body may be
-     * cached when the method is being executed.
-     */
-    private byte[] contentCache = null; 
+    private RequestEntity requestEntity;
     
     /** Counts how often the request was sent to the server. */
     private int repeatCount = 0;
 
     /** The content length of the <code>requestBodyStream</code> or one of
      *  <code>CONTENT_LENGTH_AUTO</code> and <code>CONTENT_LENGTH_CHUNKED</code>.
+     * 
+     * @deprecated
      */
-    private int requestContentLength = CONTENT_LENGTH_AUTO;
+    private long requestContentLength = InputStreamRequestEntity.CONTENT_LENGTH_AUTO;
+    
+    private boolean chunked = false;
 
     // ----------------------------------------------------------- Constructors
 
@@ -121,35 +119,6 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
         setFollowRedirects(false);
     }
 
-    /**
-     * Constructor specifying a URI and a tempDir.
-     *
-     * @param uri either an absolute or relative URI
-     * @param tempDir directory to store temp files in
-     *
-     * @deprecated the client is responsible for disk I/O
-     * @since 2.0
-     */
-    public EntityEnclosingMethod(String uri, String tempDir) {
-        super(uri, tempDir);
-        setFollowRedirects(false);
-    }
-
-    /**
-     * Constructor specifying a URI, tempDir and tempFile.
-     *
-     * @param uri either an absolute or relative URI
-     * @param tempDir directory to store temp files in
-     * @param tempFile file to store temporary data in
-     *
-     * @deprecated the client is responsible for disk I/O
-     * @since 2.0
-     */
-    public EntityEnclosingMethod(String uri, String tempDir, String tempFile) {
-        super(uri, tempDir, tempFile);
-        setFollowRedirects(false);
-    }
-
     /**
      * Returns <tt>true</tt> if there is a request body to be sent.
      * 
@@ -163,13 +132,13 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      */
     protected boolean hasRequestContent() {
         LOG.trace("enter EntityEnclosingMethod.hasRequestContent()");
-        return (this.buffer != null) 
+        return (this.requestEntity != null) 
             || (this.requestStream != null) 
             || (this.requestString != null);
     }
 
     /**
-     * Clears request body.
+     * Clears the request body.
      * 
      * <p>This method must be overridden by sub-classes that implement
      * alternative request content input methods.</p>
@@ -180,12 +149,11 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
         LOG.trace("enter EntityEnclosingMethod.clearRequestBody()");
         this.requestStream = null;
         this.requestString = null;
-        this.buffer = null;
-        this.contentCache = null;
+        this.requestEntity = null;
     }
 
     /**
-     * Generates request body.   
+     * Generates the request body.   
      * 
      * <p>This method must be overridden by sub-classes that implement
      * alternative request content input methods.</p>
@@ -197,19 +165,38 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      */
     protected byte[] generateRequestBody() {
         LOG.trace("enter EntityEnclosingMethod.renerateRequestBody()");
-        if (this.requestStream != null) {
-            bufferContent();
-        }
-         
-        if (this.buffer != null) { 
-            return this.buffer;
-        } else if (this.requestString != null) {
-            return HttpConstants.getContentBytes(this.requestString, getRequestCharSet());
-        } else {
-            return null;
-        }
+        return null;
     }
 
+    protected RequestEntity generateRequestEntity() {
+        
+        byte[] requestBody = generateRequestBody();
+        if (requestBody != null) {
+            // use the request body, if it exists.
+            // this is just for backwards compatability
+            this.requestEntity = new ByteArrayRequestEntity(requestBody);
+        } else if (this.requestStream != null) {
+            this.requestEntity = new InputStreamRequestEntity(
+                requestStream, 
+                requestContentLength);
+            this.requestStream = null;
+        } else if (this.requestString != null) {
+            String charset = getRequestCharSet(); 
+            try {
+                this.requestEntity = new StringRequestEntity(
+                        requestString, null, charset);
+            } catch (UnsupportedEncodingException e) {
+                if (LOG.isWarnEnabled()) {
+                    LOG.warn(charset + " not supported");
+                }
+                this.requestEntity = new StringRequestEntity(
+                        requestString);
+            }
+        }
+
+        return this.requestEntity;
+    }
+    
     /**
      * Entity enclosing requests cannot be redirected without user intervention
      * according to RFC 2616.
@@ -231,10 +218,7 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      */
     public void setFollowRedirects(boolean followRedirects) {
         if (followRedirects == true) {
-            // TODO: EntityEnclosingMethod should inherit from HttpMethodBase rather than GetMethod
-            // Enable exception once the inheritence is fixed
-            //throw new IllegalArgumentException(
-            //  "Entity enclosing requests cannot be redirected without user intervention");
+            throw new IllegalArgumentException("Entity enclosing requests cannot be redirected without user intervention");
         }
         super.setFollowRedirects(false);
     }
@@ -257,7 +241,9 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      *        the user is responsible to supply the correct content length.
      *        If CONTENT_LENGTH_AUTO is specified the request will be buffered
      *        before it is sent over the network.
-     *
+     * 
+     * @deprecated Use {@link #setContentChunked(boolean)} or 
+     * {@link #setRequestEntity(RequestEntity)}
      */
     public void setRequestContentLength(int length) {
         LOG.trace("enter EntityEnclosingMethod.setRequestContentLength(int)");
@@ -265,54 +251,164 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
     }
 
     /**
-     * Overrides method of {@link org.apache.commons.httpclient.HttpMethodBase}
-     * to return the length of the request body.
+     * Returns the request's charset.  The charset is parsed from the request entity's 
+     * content type, unless the content type header has been set manually. 
+     * 
+     * @see RequestEntity#getContentType()
+     * 
+     * @since 3.0
+     */
+    public String getRequestCharSet() {
+        if (getRequestHeader("Content-Type") == null) {
+            // check the content type from request entity
+            // We can't call getRequestEntity() since it will probably call
+            // this method.
+            if (this.requestEntity != null) {
+                return getContentCharSet(
+                    new Header("Content-Type", requestEntity.getContentType()));
+            } else {
+                return super.getRequestCharSet();
+            }
+        } else {
+            return super.getRequestCharSet();
+        }
+    }
+
+    /**
+     * Sets length information about the request body.
+     *
+     * <p>
+     * Note: If you specify a content length the request is unbuffered. This
+     * prevents redirection and automatic retry if a request fails the first
+     * time. This means that the HttpClient can not perform authorization
+     * automatically but will throw an Exception. You will have to set the
+     * necessary 'Authorization' or 'Proxy-Authorization' headers manually.
+     * </p>
+     *
+     * @param length size in bytes or any of CONTENT_LENGTH_AUTO,
+     *        CONTENT_LENGTH_CHUNKED. If number of bytes or CONTENT_LENGTH_CHUNKED
+     *        is specified the content will not be buffered internally and the
+     *        Content-Length header of the request will be used. In this case
+     *        the user is responsible to supply the correct content length.
+     *        If CONTENT_LENGTH_AUTO is specified the request will be buffered
+     *        before it is sent over the network.
+     * 
+     * @deprecated Use {@link #setContentChunked(boolean)} or 
+     * {@link #setRequestEntity(RequestEntity)}
+     */
+    public void setRequestContentLength(long length) {
+        LOG.trace("enter EntityEnclosingMethod.setRequestContentLength(int)");
+        this.requestContentLength = length;
+    }
+
+    /**
+     * Sets whether or not the content should be chunked.
+     * 
+     * @param chunked <code>true</code> if the content should be chunked
+     * 
+     * @since 3.0
+     */
+    public void setContentChunked(boolean chunked) {
+        this.chunked = chunked;
+    }
+    
+    /**
+     * Returns the length of the request body.
      *
      * @return number of bytes in the request body
      */
-    protected int getRequestContentLength() {
+    protected long getRequestContentLength() {
         LOG.trace("enter EntityEnclosingMethod.getRequestContentLength()");
 
         if (!hasRequestContent()) {
             return 0;
         }
-        if (this.requestContentLength != CONTENT_LENGTH_AUTO) {
-            return this.requestContentLength;
+        if (this.chunked) {
+            return -1;
         }
-        if (this.contentCache == null) {
-            this.contentCache = generateRequestBody(); 
+        if (this.requestEntity == null) {
+            this.requestEntity = generateRequestEntity(); 
         }
-        return (this.contentCache == null) ? 0 : this.contentCache.length;
+        return (this.requestEntity == null) ? 0 : this.requestEntity.getContentLength();
     }
 
     /**
-     * Generates <tt>Content-Length</tt> or <tt>Transfer-Encoding: Chunked</tt>
-     * request header, as long as no <tt>Content-Length</tt> request header
-     * already exists.
-     * 
+     * Populates the request headers map to with additional 
+     * {@link org.apache.commons.httpclient.Header headers} to be submitted to 
+     * the given {@link HttpConnection}.
+     *
+     * <p>
+     * This implementation adds tt>Content-Length</tt> or <tt>Transfer-Encoding</tt>
+     * headers.
+     * </p>
+     *
+     * <p>
+     * Subclasses may want to override this method to to add additional
+     * headers, and may choose to invoke this implementation (via
+     * <tt>super</tt>) to add the "standard" headers.
+     * </p>
+     *
      * @param state the {@link HttpState state} information associated with this method
      * @param conn the {@link HttpConnection connection} used to execute
      *        this HTTP method
      *
-     * @throws IOException if an I/O (transport) error occurs
-     * @throws HttpException  if a protocol exception occurs.
-     * @throws HttpRecoverableException if a recoverable transport error occurs. 
-     *                    Usually this kind of exceptions can be recovered from by
-     *                    retrying the HTTP method 
+     * @throws IOException if an I/O (transport) error occurs. Some transport exceptions
+     *                     can be recovered from.
+     * @throws HttpException  if a protocol exception occurs. Usually protocol exceptions 
+     *                    cannot be recovered from.
+     *
+     * @see #writeRequestHeaders
+     * 
+     * @since 3.0
+     */
+    protected void addRequestHeaders(HttpState state, HttpConnection conn)
+    throws IOException, HttpException {
+        LOG.trace("enter EntityEnclosingMethod.addRequestHeaders(HttpState, "
+            + "HttpConnection)");
+
+        super.addRequestHeaders(state, conn);
+        addContentLengthRequestHeader(state, conn);
+
+        // only use the content type of the request entity if it has not already been
+        // set manually
+        if (getRequestHeader("Content-Type") == null) {
+            RequestEntity requestEntity = getRequestEntity();
+            if (requestEntity != null && requestEntity.getContentType() != null) {
+                setRequestHeader("Content-Type", requestEntity.getContentType());
+            }
+        }
+    }
+    
+    /**
+     * Generates <tt>Content-Length</tt> or <tt>Transfer-Encoding: Chunked</tt>
+     * request header, as long as no <tt>Content-Length</tt> request header
+     * already exists.
+     *
+     * @param state current state of http requests
+     * @param conn the connection to use for I/O
+     *
+     * @throws IOException when errors occur reading or writing to/from the
+     *         connection
+     * @throws HttpException when a recoverable error occurs
      */
     protected void addContentLengthRequestHeader(HttpState state,
                                                  HttpConnection conn)
     throws IOException, HttpException {
-        LOG.trace("enter HttpMethodBase.addContentLengthRequestHeader("
+        LOG.trace("enter EntityEnclosingMethod.addContentLengthRequestHeader("
                   + "HttpState, HttpConnection)");
 
-        if ((getRequestHeader("content-length") == null)
+        if ((getRequestHeader("content-length") == null) 
             && (getRequestHeader("Transfer-Encoding") == null)) {
-            int len = getRequestContentLength();
-            if (len >= 0) {
+            long len = getRequestContentLength();
+            if (len < 0) {
+                if (getEffectiveVersion().greaterEquals(HttpVersion.HTTP_1_1)) {
+                    addRequestHeader("Transfer-Encoding", "chunked");
+                } else {
+                    throw new ProtocolException(getEffectiveVersion() + 
+                        " does not support chunk encoding");
+                }
+            } else {
                 addRequestHeader("Content-Length", String.valueOf(len));
-            } else if ((len == CONTENT_LENGTH_CHUNKED) && (isHttp11())) {
-                addRequestHeader("Transfer-Encoding", "chunked");
             }
         }
     }
@@ -321,6 +417,8 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      * Sets the request body to be the specified inputstream.
      *
      * @param body Request body content as {@link java.io.InputStream}
+     * 
+     * @deprecated use {@link #setRequestEntity(RequestEntity)}
      */
     public void setRequestBody(InputStream body) {
         LOG.trace("enter EntityEnclosingMethod.setRequestBody(InputStream)");
@@ -328,22 +426,6 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
         this.requestStream = body;
     }
 
-    /**
-     * Returns the request body as a {@link java.io.InputStream}.
-     * Calling this method will cause the content to be buffered.
-     *
-     * @return The request body as a {@link java.io.InputStream} if it has been set.
-     */
-    public InputStream getRequestBody() {
-        LOG.trace("enter EntityEnclosingMethod.getRequestBody()");
-        byte [] content = generateRequestBody();
-        if (content != null) {
-            return new ByteArrayInputStream(content);
-        } else {
-            return new ByteArrayInputStream(new byte[] {});
-        }
-    }
-
     /**
      * Sets the request body to be the specified string.
      * The string will be submitted, using the encoding
@@ -355,6 +437,8 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      * content encoding is used (ISO-8859-1).
      *
      * @param body Request body content as a string
+     * 
+     * @deprecated use {@link #setRequestEntity(RequestEntity)}
      */
     public void setRequestBody(String body) {
         LOG.trace("enter EntityEnclosingMethod.setRequestBody(String)");
@@ -362,25 +446,6 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
         this.requestString = body;
     }
 
-    /**
-     * Returns the request body as a {@link java.lang.String}.
-     * Calling this method will cause the content to be buffered.
-     *
-     * @return the request body as a {@link java.lang.String}
-     * 
-     * @throws IOException when i/o error occurs while reading the request
-     */
-    public String getRequestBodyAsString() throws IOException {
-        LOG.trace("enter EntityEnclosingMethod.getRequestBodyAsString()");
-        byte [] content = generateRequestBody();
-        if (content != null) {
-            return HttpConstants.getContentString(content, getRequestCharSet());
-        } else {
-            return null;
-        }
-    }
-
-
     /**
      * Writes the request body to the given {@link HttpConnection connection}.
      *
@@ -390,11 +455,10 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
      *
      * @return <tt>true</tt>
      *
-     * @throws IOException if an I/O (transport) error occurs
-     * @throws HttpException  if a protocol exception occurs.
-     * @throws HttpRecoverableException if a recoverable transport error occurs. 
-     *                    Usually this kind of exceptions can be recovered from by
-     *                    retrying the HTTP method 
+     * @throws IOException if an I/O (transport) error occurs. Some transport exceptions
+     *                     can be recovered from.
+     * @throws HttpException  if a protocol exception occurs. Usually protocol exceptions 
+     *                    cannot be recovered from.
      */
     protected boolean writeRequestBody(HttpState state, HttpConnection conn)
     throws IOException, HttpException {
@@ -405,66 +469,38 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
             LOG.debug("Request body has not been specified");
             return true;
         }
-
-        int contentLength = getRequestContentLength();
-
-        if ((contentLength == CONTENT_LENGTH_CHUNKED) && !isHttp11()) {
-            throw new HttpException(
-                "Chunked transfer encoding not allowed for HTTP/1.0");
+        if (this.requestEntity == null) {
+            this.requestEntity = generateRequestEntity(); 
         }
-        
-        InputStream instream = null;
-        if (this.requestStream != null) {
-            LOG.debug("Using unbuffered request body"); 
-            instream = this.requestStream;
-        } else { 
-            if (this.contentCache == null) {
-                this.contentCache = generateRequestBody();
-            }
-            if (this.contentCache != null) { 
-                LOG.debug("Using buffered request body"); 
-                instream = new ByteArrayInputStream(this.contentCache);
-            }
-        }
-
-        if (instream == null) {
+        if (requestEntity == null) {
             LOG.debug("Request body is empty");
             return true;
         }
 
-        if ((this.repeatCount > 0) && (this.contentCache == null)) {
-            throw new HttpException(
+        long contentLength = getRequestContentLength();
+
+        if ((this.repeatCount > 0) && !requestEntity.isRepeatable()) {
+            throw new ProtocolException(
                 "Unbuffered entity enclosing request can not be repeated.");
         }
 
         this.repeatCount++;
 
         OutputStream outstream = conn.getRequestOutputStream();
-
-        if (contentLength == CONTENT_LENGTH_CHUNKED) {
+        
+        if (contentLength < 0) {
             outstream = new ChunkedOutputStream(outstream);
         }
-        if (contentLength >= 0) {
-            // don't need a watcher here - we're reading from something local,
-            // not server-side.
-            instream = new ContentLengthInputStream(instream, contentLength);
-        }
-
-        byte[] tmp = new byte[4096];
-        int total = 0;
-        int i = 0;
-        while ((i = instream.read(tmp)) >= 0) {
-            outstream.write(tmp, 0, i);
-            total += i;
-        }
+        
+        requestEntity.writeRequest(outstream);
+        
         // This is hardly the most elegant solution to closing chunked stream
         if (outstream instanceof ChunkedOutputStream) {
-            ((ChunkedOutputStream) outstream).writeClosingChunk();
-        }
-        if ((contentLength > 0) && (total < contentLength)) {
-            throw new IOException("Unexpected end of input stream after "
-                + total + " bytes (expected " + contentLength + " bytes)");
+            ((ChunkedOutputStream) outstream).finish();
         }
+        
+        outstream.flush();
+        
         LOG.debug("Request body sent");
         return true;
     }
@@ -483,36 +519,29 @@ public abstract class EntityEnclosingMethod extends ExpectContinueMethod {
     public void recycle() {
         LOG.trace("enter EntityEnclosingMethod.recycle()");
         clearRequestBody();
-        this.requestContentLength = CONTENT_LENGTH_AUTO;
+        this.requestContentLength = InputStreamRequestEntity.CONTENT_LENGTH_AUTO;
         this.repeatCount = 0;
+        this.chunked = false;
         super.recycle();
     }
 
     /**
-     * Buffers request body input stream.
+     * @return Returns the requestEntity.
+     * 
+     * @since 3.0
      */
-    private void bufferContent() {
-        LOG.trace("enter EntityEnclosingMethod.bufferContent()");
-
-        if (this.buffer != null) {
-            // Already been buffered
-            return;
-        }
-        if (this.requestStream != null) {
-            try {
-                ByteArrayOutputStream tmp = new ByteArrayOutputStream();
-                byte[] data = new byte[4096];
-                int l = 0;
-                while ((l = this.requestStream.read(data)) >= 0) {
-                    tmp.write(data, 0, l);
-                }
-                this.buffer = tmp.toByteArray();
-                this.requestStream = null;
-            } catch (IOException e) {
-                LOG.error(e.getMessage(), e);
-                this.buffer = null;
-                this.requestStream = null;
-            }
-        }
+    public RequestEntity getRequestEntity() {
+        return generateRequestEntity();
     }
+
+    /**
+     * @param requestEntity The requestEntity to set.
+     * 
+     * @since 3.0
+     */
+    public void setRequestEntity(RequestEntity requestEntity) {
+        clearRequestBody();
+        this.requestEntity = requestEntity;
+    }
+
 }
