@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/java/org/apache/commons/httpclient/util/URIUtil.java,v 1.21.2.1 2004/02/22 18:21:16 olegk Exp $
- * $Revision: 1.21.2.1 $
- * $Date: 2004/02/22 18:21:16 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/java/org/apache/commons/httpclient/util/URIUtil.java,v 1.27 2004/05/05 20:34:01 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  *
  * ====================================================================
  *
@@ -25,14 +25,14 @@
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
  *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient.util;
 
-import java.io.UnsupportedEncodingException;
 import java.util.BitSet;
+
+import org.apache.commons.codec.DecoderException;
+import org.apache.commons.codec.net.URLCodec;
 import org.apache.commons.httpclient.URI;
 import org.apache.commons.httpclient.URIException;
 
@@ -42,9 +42,8 @@ import org.apache.commons.httpclient.URIException;
  * than {@link org.apache.commons.httpclient.URI}.
  *
  * @author <a href="mailto:jericho@apache.org">Sung-Gu</a>
- * @version $Revision: 1.21.2.1 $ $Date: 2002/03/14 15:14:01 
+ * @version $Revision: 155418 $ $Date: 2002/03/14 15:14:01 
  */
-
 public class URIUtil {
 
     // ----------------------------------------------------- Instance variables
@@ -480,7 +479,6 @@ public class URIUtil {
      * @throws URIException if the default protocol charset is not supported
      * 
      * @see URI#getDefaultProtocolCharset
-     * @see Coder#encode
      */
     public static String encode(String unescaped, BitSet allowed)
         throws URIException {
@@ -497,15 +495,12 @@ public class URIUtil {
      * @param allowed allowed characters not to be escaped
      * @param charset the charset
      * @return the escaped string
-     * 
-     * @throws URIException if the charset is not supported
-     * 
-     * @see Coder#encode
      */
     public static String encode(String unescaped, BitSet allowed,
             String charset) throws URIException {
-
-        return new String(Coder.encode(unescaped, allowed, charset));
+        byte[] rawdata = URLCodec.encodeUrl(allowed, 
+            EncodingUtil.getBytes(unescaped, charset));
+        return EncodingUtil.getAsciiString(rawdata);
     }
 
 
@@ -516,16 +511,19 @@ public class URIUtil {
      * @param escaped a string
      * @return the unescaped string
      * 
-     * @throws URIException if the default protocol charset is not supported
+     * @throws URIException if the string cannot be decoded (invalid)
      * 
      * @see URI#getDefaultProtocolCharset
-     * @see Coder#decode
      */
     public static String decode(String escaped) throws URIException {
-        return Coder.decode(escaped.toCharArray(), URI.getDefaultProtocolCharset());
+        try {
+            byte[] rawdata = URLCodec.decodeUrl(EncodingUtil.getAsciiBytes(escaped));
+            return EncodingUtil.getString(rawdata, URI.getDefaultProtocolCharset());
+        } catch (DecoderException e) {
+            throw new URIException(e.getMessage());
+        }
     }
 
-
     /**
      * Unescape and decode a given string regarded as an escaped string.
      *
@@ -543,121 +541,13 @@ public class URIUtil {
         return Coder.decode(escaped.toCharArray(), charset);
     }
 
-    // --------------------------------- transforming a string between charsets
-
-    /**
-     * Convert a target string to the specified character encoded string with
-     * the default protocol charset.
-     *
-     * @param target a target string
-     * @return the protocol character encoded string
-     * 
-     * @throws URIException if the default protocol charset is not supported
-     * 
-     * @see URI#getDefaultProtocolCharset
-     * 
-     * @deprecated Do not use. To be removed
-     */
-    public static String toProtocolCharset(String target) throws URIException {
-        return toUsingCharset(
-            target, 
-            URI.getDefaultDocumentCharset(), 
-            URI.getDefaultProtocolCharset());
-    }
-
-
-    /**
-     * Convert a target string to the specified character encoded string with
-     * a given protocol charset.
-     *
-     * @param target a target string
-     * @param charset the transformed protocol charset
-     * @return the protocol character encoded string
-     * 
-     * @throws URIException if the charset is not supported
-     * 
-     * @deprecated Do not use. To be removed
-     */
-    public static String toProtocolCharset(String target, String charset)
-        throws URIException {
-
-        return toUsingCharset(target, URI.getDefaultDocumentCharset(), charset);
-    }
-
-
-    /**
-     * Convert a target string to the specified character encoded string with
-     * the default document charset.
-     *
-     * @param target a target string
-     * @return the document character encoded string
-     * 
-     * @throws URIException if the default protocol charset is not supported
-     * 
-     * @see URI#getDefaultDocumentCharset
-     * 
-     * @deprecated Do not use. To be removed
-     */
-    public static String toDocumentCharset(String target) throws URIException {
-        return toUsingCharset(target, URI.getDefaultProtocolCharset(),
-                URI.getDefaultDocumentCharset());
-    }
-
-
-    /**
-     * Convert a target string to the specified character encoded string with
-     * a given document charset.
-     *
-     * @param target a target string
-     * @param charset the transformed document charset
-     * @return the document character encoded string
-     * 
-     * @throws URIException if the charset is not supported
-     * 
-     * @deprecated Do not use. To be removed
-     */
-    public static String toDocumentCharset(String target, String charset)
-        throws URIException {
-
-        return toUsingCharset(target, URI.getDefaultProtocolCharset(), charset);
-    }
-
-
-    /**
-     * Convert a target string from the <code>fromCharset</code> charset to
-     * the <code>toCharset</code> charset.
-     * <p>
-     * What if the document charset is ISO-8859-1 and the protocol charset is
-     * UTF-8, when it's read from the document part and is used in the protocol
-     * part, the use of the method will be <code>toUsingCharset(the string,
-     * "ISO-8859-1", "UTF-8")</code>.
-     *
-     * @param target a target string
-     * @param fromCharset the previous charset
-     * @param toCharset the changing charset
-     * @return the document character encoded string
-     * 
-     * @throws URIException if either of the charsets are not supported
-     * 
-     * @deprecated Do not use. To be removed
-     */
-
-    public static String toUsingCharset(String target, String fromCharset,
-            String toCharset) throws URIException {
-
-        try {
-            return new String(target.getBytes(fromCharset), toCharset);
-        } catch (UnsupportedEncodingException error) {
-            throw new URIException(URIException.UNSUPPORTED_ENCODING,
-                    error.getMessage());
-        }
-    }
-
     // ---------------------------------------------------------- Inner classes
 
     /**
      * The basic and internal utility for URI escape and character encoding and
      * decoding.
+     * 
+     * @deprecated use org.apache.commons.codec.net.URLCodec
      */
     protected static class Coder extends URI {
 
@@ -671,6 +561,8 @@ public class URIUtil {
          * @return the escaped and encoded string
          * 
          * @throws URIException if the charset is not supported
+         * 
+         * @deprecated use org.apache.commons.codec.net.URLCodec
          */
         public static char[] encode(String unescapedComponent, BitSet allowed, String charset) 
             throws URIException {
@@ -687,6 +579,8 @@ public class URIUtil {
          * @return the escaped and encoded string
          * 
          * @throws URIException if the charset is not supported
+         * 
+         * @deprecated use org.apache.commons.codec.net.URLCodec
          */
         public static String decode(char[] escapedComponent, String charset)
             throws URIException {
