@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/test/org/apache/commons/httpclient/TestHttpConnectionManager.java,v 1.8.2.5 2004/03/28 21:06:24 mbecke Exp $
- * $Revision: 1.8.2.5 $
- * $Date: 2004/03/28 21:06:24 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/test/org/apache/commons/httpclient/TestHttpConnectionManager.java,v 1.23 2004/07/17 18:58:33 mbecke Exp $
+ * $Revision: 312494 $
+ * $Date: 2005-10-09 15:53:32 -0400 (Sun, 09 Oct 2005) $
  * ====================================================================
  *
  *  Copyright 1999-2004 The Apache Software Foundation
@@ -23,31 +23,38 @@
  * individuals on behalf of the Apache Software Foundation.  For more
  * information on the Apache Software Foundation, please see
  * <http://www.apache.org/>.
- *
- * [Additional notices, if required by prior licensing conditions]
- *
  */
 
 package org.apache.commons.httpclient;
 
 import java.io.IOException;
 import java.lang.ref.WeakReference;
+import java.net.InetAddress;
+import java.net.Socket;
+import java.net.UnknownHostException;
 
 import junit.framework.Test;
 import junit.framework.TestSuite;
 
 import org.apache.commons.httpclient.methods.GetMethod;
+import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
+import org.apache.commons.httpclient.params.HttpConnectionParams;
+import org.apache.commons.httpclient.protocol.Protocol;
+import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
+import org.apache.commons.httpclient.protocol.SecureProtocolSocketFactory;
+import org.apache.commons.httpclient.server.SimpleRequest;
+import org.apache.commons.httpclient.server.SimpleResponse;
 
 /**
  * Unit tests for {@link HttpConnectionManager}.
  *
  * @author Marc A. Saegesser
- * @version $Id: TestHttpConnectionManager.java,v 1.8.2.5 2004/03/28 21:06:24 mbecke Exp $
+ * @version $Id: TestHttpConnectionManager.java 312494 2005-10-09 19:53:32Z olegk $
  */
-public class TestHttpConnectionManager extends TestLocalHostBase {
+public class TestHttpConnectionManager extends HttpClientTestBase {
 
     // ------------------------------------------------------------ Constructor
-    public TestHttpConnectionManager(String testName) {
+    public TestHttpConnectionManager(String testName) throws IOException {
         super(testName);
     }
 
@@ -66,37 +73,36 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
 
     // ----------------------------------------------------------- Test Methods
 
-    // Test the accessor methods
-    public void testMaxConnectionsAccessors() {
-        MultiThreadedHttpConnectionManager mgr = new MultiThreadedHttpConnectionManager();
-
-        // First test the default value
-        assertEquals("Default MaxConnections", 2, mgr.getMaxConnectionsPerHost());
-
-        mgr.setMaxConnectionsPerHost(10);
-        assertEquals("MaxConnections", 10, mgr.getMaxConnectionsPerHost());
-    }
-
     /**
      * Test that the ConnectMethod correctly releases connections when
      * CONNECT fails.
      */
-    public void testConnectMethodFailureRelease() {
+    public void testConnectMethodFailureRelease() throws Exception {
         
         MultiThreadedHttpConnectionManager mgr = new MultiThreadedHttpConnectionManager();
-        mgr.setMaxTotalConnections(1);
+        mgr.getParams().setIntParameter(
+            HttpConnectionManagerParams.MAX_TOTAL_CONNECTIONS, 1);
+        client.setHttpConnectionManager(mgr);
+        this.server.setHttpService(new RejectConnectService());
         
         // we're going to execute a connect method against the localhost, assuming
         // that CONNECT is not supported.  This should test the fakeResponse()
         // code on HttpMethodBase.
-        HostConfiguration hostConfiguration = new HostConfiguration();
-        hostConfiguration.setHost(getHost(), getPort(), getProtocol());
+        client.getHostConfiguration().setProxy(server.getLocalAddress(), server.getLocalPort());
+        // we must set the host to a secure destination or the CONNECT method
+        // will not be used
+        client.getHostConfiguration().setHost(
+            "notARealHost", 
+            1234, 
+            new Protocol(
+                "https", 
+                (ProtocolSocketFactory)new FakeSecureProtocolSocketFactory(), 
+                443)
+        );
         
         GetMethod get = new GetMethod("/");
         try {
-            HttpConnection connection = mgr.getConnection(hostConfiguration);
-            ConnectMethod connect = new ConnectMethod(get);
-            assertTrue(connect.execute(new HttpState(), connection) != 200);
+            assertTrue(client.executeMethod(get) != 200);
         } catch (IOException e) {
             e.printStackTrace();
             fail("Error executing connect: " + e);
@@ -105,17 +111,15 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         // this should calling releaseConnection() releases the connection
         try {
             get.releaseConnection();
-            mgr.getConnection(hostConfiguration, 1).releaseConnection();
-        } catch (HttpException e1) {
+            mgr.getConnectionWithTimeout(client.getHostConfiguration(), 1).releaseConnection();
+        } catch (ConnectTimeoutException e1) {
             fail("Connection should have been available.");
         }
         
         get = new GetMethod("/");
         
         try {
-            HttpConnection connection = mgr.getConnection(hostConfiguration);
-            ConnectMethod connect = new ConnectMethod(get);                        
-            assertTrue(connect.execute(new HttpState(), connection) != 200);
+            assertTrue(client.executeMethod(get) != 200);
         } catch (IOException e) {
             e.printStackTrace();
             fail("Error executing connect: " + e);
@@ -124,17 +128,15 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         // make sure reading the response fully releases the connection        
         try {
             get.getResponseBodyAsString();
-            mgr.getConnection(hostConfiguration, 1).releaseConnection();
-        } catch (HttpException e1) {
+            mgr.getConnectionWithTimeout(client.getHostConfiguration(), 1).releaseConnection();
+        } catch (ConnectTimeoutException e1) {
             fail("Connection should have been available.");
         }     
         
         get = new GetMethod("/");
         
         try {
-            HttpConnection connection = mgr.getConnection(hostConfiguration);
-            ConnectMethod connect = new ConnectMethod(get);                        
-            assertTrue(connect.execute(new HttpState(), connection) != 200);
+            assertTrue(client.executeMethod(get) != 200);
         } catch (IOException e) {
             e.printStackTrace();
             fail("Error executing connect: " + e);
@@ -143,8 +145,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         // make sure closing the output stream releases the connection        
         try {
             get.getResponseBodyAsStream().close();
-            mgr.getConnection(hostConfiguration, 1).releaseConnection();
-        } catch (HttpException e) {
+            mgr.getConnectionWithTimeout(client.getHostConfiguration(), 1).releaseConnection();
+        } catch (ConnectTimeoutException e) {
             fail("Connection should have been available.");
         } catch (IOException e) {
             e.printStackTrace();
@@ -188,16 +190,18 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
 
     public void testDroppedThread() throws Exception {
 
+        this.server.setHttpService(new EchoService());
+
         MultiThreadedHttpConnectionManager mthcm = new MultiThreadedHttpConnectionManager();
-        HttpClient httpClient = createHttpClient(mthcm);
+        client.setHttpConnectionManager(mthcm);
         WeakReference wr = new WeakReference(mthcm);
 
         GetMethod method = new GetMethod("/");
-        httpClient.executeMethod(method);
+        client.executeMethod(method);
         method.releaseConnection();
 
         mthcm = null;
-        httpClient = null;
+        client = null;
         method = null;
         
         System.gc();
@@ -217,9 +221,9 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     public void testWriteRequestReleaseConnection() {
 
         MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
 
-        HttpClient client = createHttpClient(connectionManager);
+        client.setHttpConnectionManager(connectionManager);
         
         GetMethod get = new GetMethod("/") {
             protected boolean writeRequestBody(HttpState state, HttpConnection conn)
@@ -239,8 +243,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         }
         
         try {
-            connectionManager.getConnection(client.getHostConfiguration(), 1);
-        } catch (HttpException e) {
+            connectionManager.getConnectionWithTimeout(client.getHostConfiguration(), 1);
+        } catch (ConnectTimeoutException e) {
             e.printStackTrace();
             fail("Connection was not released: " + e);
         }
@@ -249,12 +253,14 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     
     public void testReleaseConnection() {
 
-        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
+        this.server.setHttpService(new EchoService());
 
-        HttpClient client = createHttpClient(connectionManager);
+        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+
+        client.setHttpConnectionManager(connectionManager);
         // we shouldn't have to wait if a connection is available
-        client.setHttpConnectionFactoryTimeout( 1 );
+        client.getParams().setConnectionManagerTimeout(1);
 
         GetMethod getMethod = new GetMethod("/");
 
@@ -268,8 +274,11 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
             // this should fail quickly since the connection has not been released
             client.executeMethod(getMethod);
             fail("a httpConnection should not be available");
-        } catch (HttpException e) {            
+        } catch (ConnectTimeoutException e) {            
+        } catch (HttpException e) {
+            fail("error reading from server; " + e);
         } catch (IOException e) {
+            e.printStackTrace();
             fail("error reading from server; " + e);
         }
 
@@ -293,14 +302,16 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
      * Makes sure that a connection gets released after the content of the body
      * is read.
      */
-    public void testResponseAutoRelease() {
+    public void testResponseAutoRelease() throws Exception  {
+
+        this.server.setHttpService(new EchoService());
 
         MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
 
-        HttpClient client = createHttpClient(connectionManager);
+        client.setHttpConnectionManager(connectionManager);
         // we shouldn't have to wait if a connection is available
-        client.setHttpConnectionFactoryTimeout( 1 );
+        client.getParams().setConnectionManagerTimeout( 1 );
 
         GetMethod getMethod = new GetMethod("/");
 
@@ -333,8 +344,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     public void testConnectionReclaiming() {
         
         MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
-        connectionManager.setMaxTotalConnections(1);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+        connectionManager.getParams().setMaxTotalConnections(1);
 
         HostConfiguration host1 = new HostConfiguration();
         host1.setHost("host1", -1, "http");
@@ -349,8 +360,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         
         try {
             // the connection from host1 should be reclaimed
-            connection = connectionManager.getConnection(host2, 100);
-        } catch (HttpException e) {
+            connection = connectionManager.getConnectionWithTimeout(host2, 100);
+        } catch (ConnectTimeoutException e) {
             e.printStackTrace();
             fail("a httpConnection should have been available: " + e);
         }        
@@ -363,8 +374,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     public void testShutdownAll() {
 
         MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
-        connectionManager.setMaxTotalConnections(1);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+        connectionManager.getParams().setMaxTotalConnections(1);
 
         HostConfiguration host1 = new HostConfiguration();
         host1.setHost("host1", -1, "http");
@@ -407,8 +418,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     public void testShutdown() {
 
         MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
-        connectionManager.setMaxTotalConnections(1);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+        connectionManager.getParams().setMaxTotalConnections(1);
 
         HostConfiguration host1 = new HostConfiguration();
         host1.setHost("host1", -1, "http");
@@ -451,8 +462,8 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     public void testMaxConnections() {
         
         MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
-        connectionManager.setMaxTotalConnections(2);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+        connectionManager.getParams().setMaxTotalConnections(2);
 
         HostConfiguration host1 = new HostConfiguration();
         host1.setHost("host1", -1, "http");
@@ -465,9 +476,9 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     
         try {
             // this should fail quickly since the connection has not been released
-            connectionManager.getConnection(host2, 100);
-            fail("a httpConnection should not be available");
-        } catch (HttpException e) {
+            connectionManager.getConnectionWithTimeout(host2, 100);
+            fail("ConnectionPoolTimeoutException should not be available");
+        } catch (ConnectionPoolTimeoutException e) {
             // this should throw an exception
         }
         
@@ -477,18 +488,74 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         
         try {
             // there should be a connection available now
-            connection2 = connectionManager.getConnection(host2, 100);
-        } catch (HttpException e) {
+            connection2 = connectionManager.getConnectionWithTimeout(host2, 100);
+        } catch (ConnectionPoolTimeoutException e) {
             e.printStackTrace();
             fail("a httpConnection should have been available: " + e);
         }
     }    
 
+    /**
+     * Tests the MultiThreadedHttpConnectionManager's ability to restrict the maximum number 
+     * of connections per host.
+     */    
+    public void testMaxConnectionsPerHost() throws Exception {
+        
+        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+        connectionManager.getParams().setMaxTotalConnections(100);
+
+        HostConfiguration host1 = new HostConfiguration();
+        host1.setHost("host1", -1, "http");
+
+        HostConfiguration host2 = new HostConfiguration();
+        host2.setHost("host2", -1, "http");
+
+        HostConfiguration host3 = new HostConfiguration();
+        host3.setHost("host3", -1, "http");
+
+        connectionManager.getParams().setMaxConnectionsPerHost(host1, 3);
+        connectionManager.getParams().setMaxConnectionsPerHost(host2, 2);
+
+        // Host1
+        HttpConnection connection1 = connectionManager.getConnectionWithTimeout(host1, 1000);
+        HttpConnection connection2 = connectionManager.getConnectionWithTimeout(host1, 1000);
+        HttpConnection connection3 = connectionManager.getConnectionWithTimeout(host1, 1000);
+        try {
+            // this should fail quickly since the connection has not been released
+            connectionManager.getConnectionWithTimeout(host1, 100);
+            fail("ConnectionPoolTimeoutException should not be available");
+        } catch (ConnectionPoolTimeoutException e) {
+            // expected
+        }
+        
+        // Host2
+        connection1 = connectionManager.getConnectionWithTimeout(host2, 1000);
+        connection2 = connectionManager.getConnectionWithTimeout(host2, 1000);
+        try {
+            // this should fail quickly since the connection has not been released
+            connectionManager.getConnectionWithTimeout(host2, 100);
+            fail("ConnectionPoolTimeoutException should not be available");
+        } catch (ConnectionPoolTimeoutException e) {
+            // expected
+        }
+
+        // Host3 (should use the default per host value)
+        connection1 = connectionManager.getConnectionWithTimeout(host3, 1000);
+        try {
+            // this should fail quickly since the connection has not been released
+            connectionManager.getConnectionWithTimeout(host3, 100);
+            fail("ConnectionPoolTimeoutException should not be available");
+        } catch (ConnectionPoolTimeoutException e) {
+            // expected
+        }
+    }    
+
     public void testHostReusePreference() {
         
         final MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
-        connectionManager.setMaxTotalConnections(1);
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+        connectionManager.getParams().setMaxTotalConnections(1);
 
         final HostConfiguration host1 = new HostConfiguration();
         host1.setHost("host1", -1, "http");
@@ -534,12 +601,14 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     
     public void testMaxConnectionsPerServer() {
      
-        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
+        this.server.setHttpService(new EchoService());
 
-        HttpClient client = createHttpClient(connectionManager);
+        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
+        connectionManager.getParams().setDefaultMaxConnectionsPerHost(1);
+
+        client.setHttpConnectionManager(connectionManager);
         // we shouldn't have to wait if a connection is available
-        client.setHttpConnectionFactoryTimeout( 1 );
+        client.getParams().setConnectionManagerTimeout( 1 );
 
         GetMethod getMethod = new GetMethod("/");
 
@@ -555,21 +624,47 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
             // this should fail quickly since the connection has not been released
             client.executeMethod(getMethod2);
             fail("a httpConnection should not be available");
+        } catch (ConnectTimeoutException e) {
         } catch (HttpException e) {
+            fail("error reading from server; " + e);
         } catch (IOException e) {
             fail("error reading from server; " + e);
         }
                 
     }
     
+    public void testDeleteClosedConnections() {
+        
+        MultiThreadedHttpConnectionManager manager = new MultiThreadedHttpConnectionManager();
+        
+        HttpConnection conn = manager.getConnection(client.getHostConfiguration());
+        
+        assertEquals("connectionsInPool", manager.getConnectionsInPool(), 1);
+        assertEquals("connectionsInPool(host)", manager.getConnectionsInPool(client.getHostConfiguration()), 1);
+        
+        conn.close();
+        conn.releaseConnection();
+
+        assertEquals("connectionsInPool", manager.getConnectionsInPool(), 1);
+        assertEquals("connectionsInPool(host)", manager.getConnectionsInPool(client.getHostConfiguration()), 1);
+
+        manager.deleteClosedConnections();
+        
+        assertEquals("connectionsInPool", manager.getConnectionsInPool(), 0);
+        assertEquals("connectionsInPool(host)", manager.getConnectionsInPool(client.getHostConfiguration()), 0);
+    }
+    
     public void testReclaimUnusedConnection() {
 
-        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
-        connectionManager.setMaxConnectionsPerHost(1);
+        this.server.setHttpService(new EchoService());
 
-        HttpClient client = createHttpClient(connectionManager);
+        MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
+        connectionManager.getParams().setIntParameter(
+            HttpConnectionManagerParams.MAX_TOTAL_CONNECTIONS, 1);
+
+        client.setHttpConnectionManager(connectionManager);
         // we shouldn't have to wait if a connection is available
-        client.setHttpConnectionFactoryTimeout( 30000 );
+        client.getParams().setConnectionManagerTimeout( 30000 );
 
         GetMethod getMethod = new GetMethod("/");
 
@@ -597,7 +692,9 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
     
     public void testGetFromMultipleThreads() {
         
-        HttpClient client = createHttpClient(new MultiThreadedHttpConnectionManager());
+        this.server.setHttpService(new EchoService());
+
+        client.setHttpConnectionManager(new MultiThreadedHttpConnectionManager());
         ExecuteMethodThread[] threads = new ExecuteMethodThread[10];
         
         for (int i = 0; i < threads.length; i++) {
@@ -627,7 +724,7 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
 
     public void testTimeout() {
         MultiThreadedHttpConnectionManager mgr = new MultiThreadedHttpConnectionManager();
-        mgr.setMaxConnectionsPerHost(2);
+        mgr.getParams().setDefaultMaxConnectionsPerHost(2);
         
         try{
             HostConfiguration hostConfig = new HostConfiguration();
@@ -636,14 +733,51 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
             HttpConnection conn1 = mgr.getConnection(hostConfig);
             HttpConnection conn2 = mgr.getConnection(hostConfig);
             
-            HttpConnection conn3 = mgr.getConnection(hostConfig, 1000);
+            HttpConnection conn3 = mgr.getConnectionWithTimeout(hostConfig, 1000);
             fail("Expected an HttpException.");
             
-        }catch(HttpException e){
+        }catch(ConnectTimeoutException e){
             //Expected result
         }
     }
     
+    static class FakeSecureProtocolSocketFactory implements SecureProtocolSocketFactory {
+        
+        public Socket createSocket(Socket socket, String host, int port, boolean autoClose)
+            throws IOException, UnknownHostException {
+            throw new IllegalStateException("createSocket() should never have been called.");
+        }
+        
+        public Socket createSocket(String host, int port)
+            throws IOException, UnknownHostException {
+            throw new IllegalStateException("createSocket() should never have been called.");
+        }
+        
+        public Socket createSocket(String host, int port, InetAddress clientHost, int clientPort)
+            throws IOException, UnknownHostException {
+            throw new IllegalStateException("createSocket() should never have been called.");
+        }
+        
+        public Socket createSocket(String host, int port, InetAddress clientHost, int clientPort, 
+            HttpConnectionParams params)
+            throws IOException, UnknownHostException {
+            throw new IllegalStateException("createSocket() should never have been called.");
+        }
+    }
+    
+    static class RejectConnectService extends EchoService {
+        public boolean process(SimpleRequest request, SimpleResponse response)
+                throws IOException {
+            if (request.getRequestLine().getMethod().equalsIgnoreCase("CONNECT")) {
+                response.setStatusLine(request.getRequestLine().getHttpVersion(), HttpStatus.SC_METHOD_NOT_ALLOWED);
+                response.setHeader(new Header("Connection", "close"));
+                return true;
+            } else {
+                return super.process(request, response);
+            }
+        }
+    }
+    
     static class GetConnectionThread extends Thread {
         
         private HostConfiguration hostConfiguration;
@@ -664,9 +798,9 @@ public class TestHttpConnectionManager extends TestLocalHostBase {
         
         public void run() {
             try {
-                connection = connectionManager.getConnection(hostConfiguration, timeout);
+                connection = connectionManager.getConnectionWithTimeout(hostConfiguration, timeout);
             } catch (Exception e) {
-                exception = e;
+                this.exception = e;
             }            
         }
         
