@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvs/jakarta-commons/httpclient/src/test/org/apache/commons/httpclient/TestRequestHeaders.java,v 1.5.2.1 2004/02/22 18:21:16 olegk Exp $
- * $Revision: 1.5.2.1 $
- * $Date: 2004/02/22 18:21:16 $
+ * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//httpclient/src/test/org/apache/commons/httpclient/TestRequestHeaders.java,v 1.8 2004/10/31 14:04:13 olegk Exp $
+ * $Revision: 155418 $
+ * $Date: 2005-02-26 08:01:52 -0500 (Sat, 26 Feb 2005) $
  * ====================================================================
  *
  *  Copyright 1999-2004 The Apache Software Foundation
@@ -30,6 +30,8 @@
 
 package org.apache.commons.httpclient;
 
+import org.apache.commons.httpclient.protocol.Protocol;
+
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
@@ -38,14 +40,10 @@ import junit.framework.TestSuite;
  * Tests for reading response headers.
  *
  * @author <a href="mailto:jsdever@apache.org">Jeff Dever</a>
- * @version $Id: TestRequestHeaders.java,v 1.5.2.1 2004/02/22 18:21:16 olegk Exp $
+ * @version $Id: TestRequestHeaders.java 155418 2005-02-26 13:01:52Z dirkv $
  */
 public class TestRequestHeaders extends TestCase {
 
-       HttpState state = null;
-       SimpleHttpMethod method = null;
-       HttpConnection conn = null;
-
     // ------------------------------------------------------------ Constructor
     public TestRequestHeaders(String testName) {
         super(testName);
@@ -62,49 +60,48 @@ public class TestRequestHeaders extends TestCase {
         return new TestSuite(TestRequestHeaders.class);
     }
 
-    public void setUp() {
-       state = new HttpState();
-       method = new SimpleHttpMethod("/some/absolute/path/");
-       //assign conn in test case
-    }
-
-    public void tearDown() {
-       state = null;
-       method = null;
-       conn = null;
-    }
-
     public void testNullHeader() throws Exception {
-       conn = new SimpleHttpConnection("some.host.name", 80, "http");
-       assertEquals(null, method.getRequestHeader(null));
-       assertEquals(null, method.getRequestHeader("bogus"));
+        FakeHttpMethod method = new FakeHttpMethod();
+        assertEquals(null, method.getRequestHeader(null));
+        assertEquals(null, method.getRequestHeader("bogus"));
     }
 
     public void testHostHeaderPortHTTP80() throws Exception {
-       conn = new SimpleHttpConnection("some.host.name", 80, "http");
-       method.addRequestHeaders(state, conn);
-       assertEquals("Host: some.host.name", method.getRequestHeader("Host").toString().trim());
+        HttpConnection conn = new HttpConnection("some.host.name", 80);
+        HttpState state = new HttpState();
+        FakeHttpMethod method = new FakeHttpMethod();
+        method.addRequestHeaders(state, conn);
+        assertEquals("Host: some.host.name", method.getRequestHeader("Host").toString().trim());
     }
 
     public void testHostHeaderPortHTTP81() throws Exception {
-       conn = new SimpleHttpConnection("some.host.name", 81, "http");
-       method.addRequestHeaders(state, conn);
-       assertEquals("Host: some.host.name:81", method.getRequestHeader("Host").toString().trim());
+        HttpConnection conn = new HttpConnection("some.host.name", 81);
+        HttpState state = new HttpState();
+        FakeHttpMethod method = new FakeHttpMethod();
+        method.addRequestHeaders(state, conn);
+        assertEquals("Host: some.host.name:81", method.getRequestHeader("Host").toString().trim());
     }
 
     public void testHostHeaderPortHTTPS443() throws Exception {
-       conn = new SimpleHttpConnection("some.host.name", 443, "https");
-       method.addRequestHeaders(state, conn);
-       assertEquals("Host: some.host.name", method.getRequestHeader("Host").toString().trim());
+        HttpConnection conn = new HttpConnection("some.host.name", 443, 
+                Protocol.getProtocol("https"));
+        HttpState state = new HttpState();
+        FakeHttpMethod method = new FakeHttpMethod();
+        method.addRequestHeaders(state, conn);
+        assertEquals("Host: some.host.name", method.getRequestHeader("Host").toString().trim());
     }
 
     public void testHostHeaderPortHTTPS444() throws Exception {
-       conn = new SimpleHttpConnection("some.host.name", 444, "https");
-       method.addRequestHeaders(state, conn);
-       assertEquals("Host: some.host.name:444", method.getRequestHeader("Host").toString().trim());
+        HttpConnection conn = new HttpConnection("some.host.name", 444, 
+                Protocol.getProtocol("https"));
+        HttpState state = new HttpState();
+        FakeHttpMethod method = new FakeHttpMethod();
+        method.addRequestHeaders(state, conn);
+        assertEquals("Host: some.host.name:444", method.getRequestHeader("Host").toString().trim());
     }
 
     public void testHeadersPreserveCaseKeyIgnoresCase() throws Exception {
+        FakeHttpMethod method = new FakeHttpMethod();
         method.addRequestHeader(new Header("NAME", "VALUE"));
         Header upHeader =  method.getRequestHeader("NAME");
         Header loHeader =  method.getRequestHeader("name");
